@@ -36,8 +36,12 @@ execute()
 	file="$4"
 	smil_file="$5"
 	pass="$6"
+
+	source "`dirname $0`/ffmpeg_utils.sh"
+
+	# generate filename if missing
 	[ "x$file" = "x" ] && file="kino_export_"`date +%Y-%m-%d_%H.%M.%S`
-	file="`dirname \"$4\"`/`basename \"$4\"`"
+	file="`dirname \"$file\"`/`basename \"$file\"`"
 
 	# Determine info arguments
 	frame_rate=`[ "$normalisation" = "pal" ] && echo 25 || echo 29.97`
