@@ -1,6 +1,6 @@
 #!/bin/sh
 
-GNOME_BROWSER=`gconftool --get /desktop/gnome/applications/browser/exec`
+GNOME_BROWSER=`gconftool-2 --get /desktop/gnome/url-handlers/http/command | sed s/%s//`
 which konqueror &> /dev/null
 [ $? = 0 ] && KONQUEROR=konqueror
 which mozilla &> /dev/null
