@@ -49,7 +49,7 @@ using std::endl;
 
 #define PADDING_SIZE (512)
 #define PADDING_1GB (0x40000000)
-#define IX00_INDEX_SIZE (17895)
+#define IX00_INDEX_SIZE (4028)
 #define IDX1_INDEX_SIZE (20000)
 
 #define AVIF_HASINDEX 0x00000010
@@ -328,8 +328,8 @@ int AVIFile::GetDVFrameInfo( off_t &offset, int &size, int frameNum )
 
 		if ( frameNum < ix[ 0 ] ->nEntriesInUse )
 		{
-			fail_if( frameNum > IX00_INDEX_SIZE );
-			offset = ix[ 0 ] ->qwBaseOffset + ix[ 0 ] ->aIndex[ frameNum ].dwOffset;
+			fail_if( unsigned( frameNum ) >= ( sizeof( ix[ 0 ]->aIndex ) / sizeof( DWORD ) / 2 ) );
+			offset = ix[ 0 ] ->qwBaseOffset + ix[ 0 ]->aIndex[ frameNum ].dwOffset;
 			size = ix[ 0 ] ->aIndex[ frameNum ].dwSize;
 			return 0;
 		}
@@ -1133,7 +1133,7 @@ void AVI1File::Init( int format, int sampleFrequency, int indexType )
 	   several RIFF chunks of less than 1 GByte size. Old software
 	   however would only deal with the first RIFF chunk.
 
-	   Note that the first entry (FILE) isnt actually part
+	   Note that the first entry (FILE) isn't actually part
 	   of the AVI file. I use this (pseudo-) directory entry to
 	   keep track of the RIFF chunks and their positions in the
 	   AVI file.
