@@ -584,7 +584,8 @@ char * Export::formatSecs( char * buf, int size, double seconds )
 }
 
 /** innerLoopUpdate - see header file. */
-void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every )
+void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every,
+		const char* activity )
 {
 	char buf[ 512 ];
 	struct timeval tv;
@@ -600,7 +601,7 @@ void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every )
 		/* First time */
 		startTime = now;
 		nextUpdateTime = now - 1;
-		snprintf( buf, 512, _( "Exporting frame %i." ), currentFrame );
+		snprintf( buf, 512, _( "%s frame %i." ), activity, currentFrame );
 	}
 	else
 	{
@@ -612,7 +613,8 @@ void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every )
 		char buf1[ 16 ];
 		char buf2[ 16 ];
 		char buf3[ 16 ];
-		snprintf( buf, 512, _( "Exporting frame %i. Time  used: %s,  estimated: %s,  left: %s" ),
+		snprintf( buf, 512, _( "%s frame %i. Time  used: %s,  estimated: %s,  left: %s" ),
+		          activity,
 		          currentFrame + 1,
 		          formatSecs( buf1, 16, time_so_far ),
 		          formatSecs( buf2, 16, total_est ),
@@ -644,3 +646,68 @@ void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every )
 		pauseTime += ( foo - now );
 	}
 }
+
+double Export::calculateAdjustedRate( PlayList* playlist, double outputRate, int begin, int end, int every )
+{
+	Frame& frame = *GetFramePool()->GetFrame();
+	playlist->GetFrame( begin, frame );
+	double seconds = double( end - begin + 1 ) / frame.GetFrameRate();
+	int64_t idealSamples = int64_t( seconds * outputRate + 0.5 );
+	int64_t actualSamples = 0;
+	double adjustedRate = 0.0;
+	struct timeval tv;
+	if ( 0 != gettimeofday( &tv, NULL ) )
+	{
+		cerr << ">>> Export::calculateAdjustedRate: Error calling gettimeofday?" << endl;
+	}
+	double now = tv.tv_sec + tv.tv_usec / 1000000.0;
+	double nextUpdateTime = now - 1.0;
+	char buf[ 512 ];
+	int n = -1, frameNum = 0;
+	AsyncAudioResample<int16_ne_t,int16_le_t>* resampler = new AsyncAudioResample<int16_ne_t,int16_le_t>(
+		AUDIO_RESAMPLE_SRC_SINC_FASTEST, playlist, outputRate, begin, end, every );
+
+	// Determine the actual number of samples
+	while ( exportPage->isExporting && n != 0 )
+	{
+		n = resampler->Process( outputRate, frame.CalculateNumberSamples( int(outputRate), frameNum++ ) );
+		actualSamples += n;
+
+		if ( 0 == gettimeofday( &tv, NULL ) )
+		{
+			now = tv.tv_sec + tv.tv_usec / 1000000.0;
+			if ( now > nextUpdateTime )
+			{
+				snprintf( buf, 512, "%s (%02d%%)", _("Locking audio"),
+					int( 100.0 * frameNum / (end - begin + 1) ) );
+				common->setStatusBar( buf );
+				nextUpdateTime = now + 1.0;
+			}
+		}
+		exportPage->updateProgress( -1 );
+
+		/* Check pause - make sure we ignore time paused in export. */
+		if ( EXPORT_PAUSE & actions && exportPage->isPausing )
+		{
+			doPause();
+		}
+	}
+	GetFramePool()->DoneWithFrame( &frame );
+
+	if ( exportPage->isExporting )
+	{
+		// If there are less actual samples than ideal then increase the
+		// rate to make resampler increase the number of samples.
+// 		cerr << "calcAdjustedRate secs " << seconds << " ideal " << idealSamples << " actual " << actualSamples;
+		adjustedRate = outputRate + ( idealSamples - actualSamples ) / seconds;
+	}
+
+	if ( 0 != gettimeofday( &tv, NULL ) )
+		cerr << ">>> Export::innerLoopUpdate: Error calling gettimeofday?" << endl;
+	double foo = tv.tv_sec + tv.tv_usec / 1000000.0;
+	pauseTime += ( foo - now );
+
+	cerr << ">>> output rate is " << outputRate << ", adjusted rate is " << adjustedRate << endl;
+
+	return adjustedRate;
+}
