@@ -767,6 +767,7 @@ int QtHandler::Write( const Frame& frame )
 	if ( ! isFullyInitialized )
 	{
 		AudioInfo audio;
+		const char* fourcc = frame.IsPAL() ? "dvcp" : QUICKTIME_DV;
 
 		if ( frame.GetAudioInfo( audio ) )
 		{
@@ -779,7 +780,7 @@ int QtHandler::Write( const Frame& frame )
 		}
 
 		quicktime_set_video( fd, 1, 720, frame.IsPAL() ? 576 : 480,
-		                     frame.GetFrameRate(), QUICKTIME_DV );
+		                     frame.GetFrameRate(), const_cast<char*>( fourcc ) );
 		AllocateAudioBuffers();
 	}
 
@@ -830,7 +831,7 @@ int QtHandler::Close()
 off_t QtHandler::GetFileSize()
 {
 	struct stat file_status;
-	fstat( fileno( fd->stream ), &file_status );
+	stat( filename.c_str(), &file_status );
 	return file_status.st_size;
 }
 
@@ -859,7 +860,12 @@ bool QtHandler::Open( const char *s )
 		Close();
 		return false;
 	}
-	if ( strncmp( quicktime_video_compressor( fd, 0 ), QUICKTIME_DV, 4 ) != 0 )
+	const char* fourcc = quicktime_video_compressor( fd, 0 );
+	if ( strncmp( fourcc, QUICKTIME_DV, 4 ) != 0 &&
+	     strncmp( fourcc, "dv25", 4 ) != 0 &&
+	     strncmp( fourcc, "dvsd", 4 ) != 0 &&
+	     strncmp( fourcc, "dvcp", 4 ) != 0 &&
+	     strncmp( fourcc, "AVdv", 4 ) != 0 )
 	{
 		fprintf( stderr, "Video in input file (%s) must be in DV format.\n", s );
 		Close();
