@@ -191,7 +191,6 @@ Frame::~Frame()
 
 #ifdef HAVE_LIBDV
 	dv_decoder_free( decoder );
-	fclose( libdv_log );
 	if ( encoder )
 		dv_encoder_free( encoder );
 #endif
@@ -201,6 +200,17 @@ Frame::~Frame()
 }
 
 
+Frame& Frame::operator=( const Frame& rhs )
+{
+	bytesInFrame = rhs.GetFrameSize();
+	if ( bytesInFrame > 144000 )
+		bytesInFrame = 144000;
+	memcpy( data, rhs.data, bytesInFrame );
+	ExtractHeader();
+	return *this;
+}
+
+
 /** gets a subcode data packet
  
     This function returns a SSYB packet from the subcode data section.
