@@ -38,7 +38,7 @@
 
 #undef FANCY
 #define ENHANCED_SCALE_CLASS(w)  GTK_ENHANCED_SCALE_CLASS ( G_OBJECT_GET_CLASS( G_OBJECT( w )  ) ) 
-/*#define DEBUG*/
+// #define DEBUG
 
 static void gtk_enhanced_scale_class_init ( GtkEnhancedScaleClass *klass );
 static void gtk_enhanced_scale_init ( GtkEnhancedScale *enhanced_scale );
@@ -1182,14 +1182,18 @@ static void gtk_enhanced_scale_adjustment_value_changed( GtkAdjustment *adjustme
 	g_return_if_fail ( adjustment != NULL );
 	g_return_if_fail ( data != NULL );
 
+#ifdef DEBUG
+	printf( "gtk_enhanced_scale_adjustment_value_changed: %p\n", adjustment );
+#endif
 	enhanced_scale = GTK_ENHANCED_SCALE( data );
+	gtk_enhanced_scale_draw ( GTK_WIDGET( enhanced_scale ), NULL );
+	return;
 
 	for ( i = 0;i < enhanced_scale->num_adjustments;i++ )
 	{
 		if ( adjustment == GTK_ADJUSTMENT( enhanced_scale->adjustment[i] ) )
 		{
 			gtk_enhanced_scale_slider_update( enhanced_scale, i );
-			gtk_enhanced_scale_draw ( GTK_WIDGET( enhanced_scale ), NULL );
 			break;
 		}
 	}
