@@ -61,9 +61,9 @@ using std::endl;
 #include <sys/mman.h>
 #include <sys/poll.h>
 #include <errno.h>
+#include <time.h>
+#include <sys/time.h>
 
-#include <libraw1394/raw1394.h>
-#include <libraw1394/csr.h>
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
 #include <libavc1394/rom1394.h>
@@ -93,6 +93,7 @@ using std::endl;
 
 
 IEEE1394Reader::IEEE1394Reader( int c, int bufSize ) :
+	droppedFrames( 0 ),
 	currentFrame( NULL ),
 	channel( c ),
 	isRunning( false )
@@ -112,9 +113,6 @@ IEEE1394Reader::IEEE1394Reader( int c, int bufSize ) :
 	/* Initialise mutex and condition for action triggerring */
 	pthread_mutex_init( &condition_mutex, NULL );
 	pthread_cond_init( &condition, NULL );
-
-	/* Other */
-	droppedFrames = 0;
 }
 
 
@@ -197,6 +195,19 @@ void IEEE1394Reader::DoneWithFrame( Frame* frame )
 }
 
 
+/** Return the number of dropped frames since last call
+*/
+
+int IEEE1394Reader::GetDroppedFrames( void )
+{
+	pthread_mutex_lock( &mutex );
+	int n = droppedFrames;
+	droppedFrames = 0;
+	pthread_mutex_unlock( &mutex );
+	return n;
+}
+
+
 /** Throw away all currently available frames.
  
     All frames in the outFrames queue are put back to the inFrames
@@ -222,7 +233,7 @@ void IEEE1394Reader::Flush()
 	pthread_mutex_unlock( &mutex );
 }
 
-bool IEEE1394Reader::WaitForAction( )
+bool IEEE1394Reader::WaitForAction( int seconds )
 {
 	pthread_mutex_lock( &mutex );
 	int size = outFrames.size( );
@@ -231,16 +242,39 @@ bool IEEE1394Reader::WaitForAction( )
 	if ( size == 0 )
 	{
 		pthread_mutex_lock( &condition_mutex );
-		pthread_cond_wait( &condition, &condition_mutex );
-		pthread_mutex_unlock( &condition_mutex );
-		pthread_mutex_lock( &mutex );
-		size = outFrames.size( );
+		if ( seconds == 0 )
+		{
+			pthread_cond_wait( &condition, &condition_mutex );
+			pthread_mutex_unlock( &condition_mutex );
+			pthread_mutex_lock( &mutex );
+			size = outFrames.size( );
+		}
+		else
+		{
+			struct timeval tp;
+			struct timespec ts;
+			int result;
+
+			gettimeofday( &tp, NULL );
+			ts.tv_sec = tp.tv_sec + seconds;
+			ts.tv_nsec = tp.tv_usec * 1000;
+
+			result = pthread_cond_timedwait( &condition, &condition_mutex, &ts );
+			pthread_mutex_unlock( &condition_mutex );
+			pthread_mutex_lock( &mutex );
+
+			if ( result == ETIMEDOUT )
+				size = 0;
+			else
+				size = outFrames.size();
+		}
 		pthread_mutex_unlock( &mutex );
 	}
 
 	return size != 0;
 }
 
+
 void IEEE1394Reader::TriggerAction( )
 {
 	pthread_mutex_lock( &condition_mutex );
@@ -249,28 +283,30 @@ void IEEE1394Reader::TriggerAction( )
 }
 
 
+#ifdef HAVE_IEC61883
 /** Initializes the raw1394Reader object.
  
     The object is initialized with port and channel number. These
     parameters define the interface card and the iso channel on which
     the camcorder sends its data.
  
-    \param p the number of the interface card to use
     \param c the iso channel number to use
     \param bufSize the number of frames to allocate for the frames buffer
  */
 
 
-raw1394Reader::raw1394Reader( int p, int c, int bufSize ) :
-		IEEE1394Reader( c, bufSize ), port( p )
+iec61883Reader::iec61883Reader( int c, int bufSize ) :
+	IEEE1394Reader( c, bufSize )
 {
-	handle = NULL;
-	cerr << ">>> Using raw1394 capture" << endl;
+	m_handle = NULL;
+	cerr << ">>> Using iec61883 capture" << endl;
 }
 
 
-raw1394Reader::~raw1394Reader()
-{}
+iec61883Reader::~iec61883Reader()
+{
+	StopThread();
+}
 
 
 /** Start receiving DV frames
@@ -281,13 +317,17 @@ raw1394Reader::~raw1394Reader()
  
 */
 
-bool raw1394Reader::StartThread()
+bool iec61883Reader::StartThread( int port )
 {
 	if ( isRunning )
 		return true;
+	if ( port < 0 )
+		return false;
+	cerr << ">>> iec61883Reader::StartThread on port " << port << endl;
+
 	pthread_mutex_lock( &mutex );
 	currentFrame = NULL;
-	if ( Open() && StartReceive() )
+	if ( Open( port ) && StartReceive() )
 	{
 		isRunning = true;
 		pthread_create( &thread, NULL, ThreadProxy, this );
@@ -310,18 +350,34 @@ bool raw1394Reader::StartThread()
     canceled, we turn off iso receive and close the ieee1394
     subsystem.  We also remove all frames in the outFrames queue that
     have not been processed until now.
+ 
 */
 
-void raw1394Reader::StopThread()
+void iec61883Reader::StopThread()
 {
 	if ( isRunning )
 	{
 		isRunning = false;
 		pthread_join( thread, NULL );
-		StopReceive();
-		Close();
-		Flush();
 	}
+	Close();
+	Flush();
+}
+
+
+void iec61883Reader::ResetHandler( void )
+{
+}
+
+int iec61883Reader::ResetHandlerProxy( raw1394handle_t handle, unsigned int generation )
+{
+	iec61883_dv_t dv = static_cast< iec61883_dv_t >( raw1394_get_userdata( handle ) );
+	iec61883_dv_fb_t dvfb = static_cast< iec61883_dv_fb_t >( iec61883_dv_get_callback_data( dv ) );
+	iec61883Reader *self = static_cast< iec61883Reader* >( iec61883_dv_fb_get_callback_data( dvfb ) );
+	raw1394_update_generation( handle, generation );
+	if ( self )
+		self->ResetHandler();
+	return 0;
 }
 
 
@@ -330,30 +386,21 @@ void raw1394Reader::StopThread()
     \return success/failure
 */
 
-bool raw1394Reader::Open()
+bool iec61883Reader::Open( int port )
 {
 	bool success;
-	int numcards;
-	struct raw1394_portinfo pinf[ 16 ];
-	iso_handler_t oldhandler;
 
-	assert( handle == NULL );
+	assert( !m_handle );
 
 	try
 	{
-
-		handle = raw1394_new_handle();
-		if ( handle == 0 )
+		m_handle = raw1394_new_handle_on_port( port );
+		if ( !m_handle )
 			return false;
-		fail_neg( numcards = raw1394_get_port_info( handle, pinf, 16 ) );
-		if ( numcards == 0 )
-			return false;
-		fail_neg( raw1394_set_port( handle, port ) );
+		raw1394_set_bus_reset_handler( m_handle, this->ResetHandlerProxy );
 
-		oldhandler = raw1394_set_iso_handler( handle, channel, HandlerProxy );
-		raw1394_set_userdata( handle, this );
-
-		success = true;
+		m_iec61883dv = iec61883_dv_fb_init( m_handle, HandlerProxy, this );
+		success = ( m_iec61883dv != NULL );
 	}
 	catch ( string exc )
 	{
@@ -369,23 +416,24 @@ bool raw1394Reader::Open()
  
 */
 
-void raw1394Reader::Close()
+void iec61883Reader::Close()
 {
-	if ( handle != NULL )
+	if ( m_handle )
 	{
-		raw1394_destroy_handle( handle );
-		handle = NULL;
+		StopReceive();
+		raw1394_destroy_handle( m_handle );
+		m_handle = NULL;
 	}
 }
 
-bool raw1394Reader::StartReceive()
+bool iec61883Reader::StartReceive()
 {
-	bool	success;
+	bool success;
 
 	/* Starting iso receive */
 	try
 	{
-		fail_neg( raw1394_start_iso_rcv( handle, channel ) );
+		fail_neg( iec61883_dv_fb_start( m_iec61883dv, channel ) );
 		success = true;
 	}
 	catch ( string exc )
@@ -397,152 +445,51 @@ bool raw1394Reader::StartReceive()
 }
 
 
-void raw1394Reader::StopReceive()
+void iec61883Reader::StopReceive()
 {
-	if ( handle != NULL )
-		raw1394_stop_iso_rcv( handle, channel );
-}
-
-
-/** assembles a DIF sequence to a DV frame
- 
-    The PAL/NTSC DV data has the following format:
- 
-    - packets of 496 bytes length
-    - packets of  16 bytes length.
- 
-    The long packets contain the actual video and audio contents that
-    goes into the AVI file.
-       
-    The actual DV data starts at quadlet 4 of the long packet, so we
-    have 480 bytes of DV data per packet.  For PAL, each rame is made
-    out of 300 packets and there are 25 frames per second.  That is
-    144000 bytes per frame and 3600000 bytes per second.  For NTSC we
-    have 250 packages per frame and 30 frames per second.  That is
-    120000 bytes per frame and 3600000 bytes per second too.
- 
-    We also attempt to detect incomplete frames. The idea is: If we
-    hit the begin of frame indicator and this is not the very first
-    packet for this frame, then at least one packed has been dropped
-    by the driver. This does not guarantee that no incomplete frames
-    are saved, because if we miss the frame header of the next frame,
-    we can´t tell whether the last one is incomplete.
- 
-    \param handle the handle to the ieee1394 subsystem
-    \param channel the channel this data has been read from
-    \param length the length of the data
-    \param data a pointer to the data
-    \return unused, we return always zero 
- 
-*/
-
-int raw1394Reader::HandlerProxy( raw1394handle_t handle, int channel, size_t length, quadlet_t *data )
-{
-	raw1394Reader *self = static_cast< raw1394Reader* >( raw1394_get_userdata( handle ) );
-	return self->Handler( channel, length, data );
-}
-
-int raw1394Reader::Handler( int channel, size_t length, quadlet_t *data )
-{
-	/* skip empty packets */
-
-	if ( length > 16 )
+	if ( m_iec61883dv )
 	{
+		iec61883_dv_fb_close( m_iec61883dv );
+		m_iec61883dv = NULL;
+	}
+}
 
-		unsigned char * p = ( unsigned char* ) & data[ 3 ];
-		int section_type = p[ 0 ] >> 5;           /* section type is in bits 5 - 7 */
-		int dif_sequence = p[ 1 ] >> 4;           /* dif sequence number is in bits 4 - 7 */
-		int dif_block = p[ 2 ];
+int iec61883Reader::HandlerProxy( unsigned char *data, int length, int complete, 
+	void *callback_data )
+{
+	iec61883Reader *self = static_cast< iec61883Reader* >( callback_data );
+	return self->Handler( length, complete, data );
+}
 
-		static int packet_count = 0;
-		static int packet_target = 300;
+int iec61883Reader::Handler( int length, int complete, unsigned char *data )
+{
+	pthread_mutex_lock( &mutex );
+	if ( currentFrame )
+	{
+		outFrames.push_back( currentFrame );
+		currentFrame = NULL;
+		//cerr << "reader > # inFrames: " << inFrames.size() << ", # outFrames: " << outFrames.size() << endl;
+		TriggerAction( );
+	}
+	if ( inFrames.size() > 0 )
+	{
+		currentFrame = inFrames.front();
+		currentFrame->bytesInFrame = 0;
+		inFrames.pop_front();
+		if ( outFrames.size( ) >= 25 )
+			cerr << "reader < # inFrames: " << inFrames.size() << ", # outFrames: " << outFrames.size() << endl;
+	}
+	else
+	{
+		droppedFrames++;
+		cerr << "reader < # dropped frames: " << droppedFrames << endl;
+	}
+	pthread_mutex_unlock( &mutex );
 
-		/* if we are at the beginning of a frame, we put the previous
-		   frame in our output_queue.  Then we try to get an unused
-		   frame_buffer from the buffer_queue for the current frame.
-		   We must lock the queues because they are shared between
-		   this thread and the main thread. */
-
-		if ( packet_count == packet_target && section_type == 0 && dif_sequence == 0 )
-		{
-			pthread_mutex_lock( &mutex );
-			if ( currentFrame != NULL )
-			{
-				outFrames.push_back( currentFrame );
-				currentFrame = NULL;
-				/* cerr << "reader > # inFrames: " << inFrames.size() << ", # outFrames: " << outFrames.size() << endl; */
-				TriggerAction( );
-			}
-			if ( inFrames.size() > 0 )
-			{
-				currentFrame = inFrames.front();
-				currentFrame->bytesInFrame = 0;
-				inFrames.pop_front();
-				if ( outFrames.size( ) >= 25 )
-					cerr << "reader < # inFrames: " << inFrames.size() << ", # outFrames: " << outFrames.size() << endl;
-			}
-			else
-			{
-				droppedFrames++;
-				cerr << "reader < # dropped frames: " << droppedFrames << endl;
-			}
-
-			packet_count = 0;
-			pthread_mutex_unlock( &mutex );
-		}
-		else if ( packet_count != 0 && section_type == 0 && dif_sequence == 0 )
-		{
-			currentFrame = inFrames.front();
-			currentFrame->bytesInFrame = 0;
-			inFrames.pop_front();
-			droppedFrames++;
-			cerr << "reader < # incomplete dropped frames: " << droppedFrames << endl;
-			packet_count = 0;
-		}
-
-		// Determine if we're looking for 300 packets (PAL) or 250 packets (NTSC)
-		if ( packet_count == 0 )
-		{
-			if ( p[ 3 ] & 0x80 )
-				packet_target = 300;
-			else
-				packet_target = 250;
-		}
-
-		// Increment the packet count
-		packet_count ++;
-
-		if ( currentFrame != NULL )
-		{
-
-			switch ( section_type )
-			{
-			case 0:   /* 1 Header block */
-				/* p[3] |= 0x80; // hack to force PAL data */
-				memcpy( currentFrame->data + dif_sequence * 150 * 80, p, 480 );
-				break;
-
-			case 1:   /* 2 Subcode blocks */
-				memcpy( currentFrame->data + dif_sequence * 150 * 80 + ( 1 + dif_block ) * 80, p, 480 );
-				break;
-
-			case 2:   /* 3 VAUX blocks */
-				memcpy( currentFrame->data + dif_sequence * 150 * 80 + ( 3 + dif_block ) * 80, p, 480 );
-				break;
-
-			case 3:   /* 9 Audio blocks interleaved with video */
-				memcpy( currentFrame->data + dif_sequence * 150 * 80 + ( 6 + dif_block * 16 ) * 80, p, 480 );
-				break;
-
-			case 4:   /* 135 Video blocks interleaved with audio */
-				memcpy( currentFrame->data + dif_sequence * 150 * 80 + ( 7 + ( dif_block / 15 ) + dif_block ) * 80, p, 480 );
-				break;
-
-			default:   /* we can´t handle any other data */
-				break;
-			}
-			currentFrame->bytesInFrame += 480;
-		}
+	if ( currentFrame )
+	{
+		memcpy( currentFrame->data, data, length );
+		currentFrame->bytesInFrame = length;
 	}
 
 	return 0;
@@ -555,18 +502,19 @@ int raw1394Reader::Handler( int channel, size_t length, quadlet_t *data )
     but only in the pthread_testcancel() function.
  
 */
-void* raw1394Reader::ThreadProxy( void* arg )
+void* iec61883Reader::ThreadProxy( void* arg )
 {
-	raw1394Reader* self = static_cast< raw1394Reader* >( arg );
+	iec61883Reader* self = static_cast< iec61883Reader* >( arg );
 	return self->Thread();
 }
 
-void* raw1394Reader::Thread()
+void* iec61883Reader::Thread()
 {
 	struct pollfd raw1394_poll;
 	int result;
-		
-	raw1394_poll.fd = raw1394_get_fd( handle );
+
+	assert( m_handle );
+	raw1394_poll.fd = raw1394_get_fd( m_handle );
 	raw1394_poll.events = POLLIN | POLLERR | POLLHUP | POLLPRI;
 
 	while ( isRunning )
@@ -581,13 +529,14 @@ void* raw1394Reader::Thread()
 		}
 		if ( result > 0 && ( ( raw1394_poll.revents & POLLIN )
 		        || ( raw1394_poll.revents & POLLPRI ) ) )
-			result = raw1394_loop_iterate( handle );
+			result = raw1394_loop_iterate( m_handle );
 	}
 	return NULL;
 }
+#endif
 
 
-
+#ifdef HAVE_DV1394
 /** Initializes the dv1394Reader object.
  
     \param c the iso channel number to use
@@ -616,13 +565,13 @@ dv1394Reader::~dv1394Reader()
  
 */
 
-bool dv1394Reader::StartThread()
+bool dv1394Reader::StartThread( int port )
 {
 	if ( isRunning )
 		return true;
 	pthread_mutex_lock( &mutex );
 	currentFrame = NULL;
-	if ( Open() && StartReceive() )
+	if ( Open( port ) && StartReceive() )
 	{
 		isRunning = true;
 		pthread_create( &thread, NULL, ThreadProxy, this );
@@ -666,7 +615,7 @@ void dv1394Reader::StopThread()
     \return success/failure
 */
 
-bool dv1394Reader::Open()
+bool dv1394Reader::Open( int port )
 {
 	int n_frames = DV1394_MAX_FRAMES / 4;
 	struct dv1394_init init =
@@ -845,6 +794,7 @@ void* dv1394Reader::Thread( )
 	}
 	return NULL;
 }
+#endif
 
 
 /** Initializes the AVC object.
@@ -853,30 +803,20 @@ void* dv1394Reader::Thread( )
  */
 
 
-AVC::AVC( int p ) : port( p )
+AVC::AVC( void ) : port( -1 ), totalPorts( 0 )
 {
 	pthread_mutex_init( &avc_mutex, NULL );
 	avc_handle = NULL;
-	int numcards;
 	struct raw1394_portinfo pinf[ 16 ];
 
 	try
 	{
-
 		avc_handle = raw1394_new_handle();
 		if ( avc_handle == 0 )
 			return;
-		fail_neg( numcards = raw1394_get_port_info( avc_handle, pinf, 16 ) );
-		if ( numcards == 0 )
-		{
-			raw1394_destroy_handle( avc_handle );
-			avc_handle = NULL;
-		}
-		else
-		{
-			fail_neg( raw1394_set_port( avc_handle, port ) );
-			raw1394_set_bus_reset_handler( avc_handle, this->ResetHandler );
-		}
+		fail_neg( totalPorts = raw1394_get_port_info( avc_handle, pinf, 16 ) );
+		raw1394_destroy_handle( avc_handle );
+		avc_handle = NULL;
 	}
 	catch ( string exc )
 	{
@@ -911,7 +851,7 @@ int AVC::ResetHandler( raw1394handle_t handle, unsigned int generation )
 {
 	cerr << "Reset Handler received" << endl;
 	raw1394_update_generation( handle, generation );
-	common->getPageCapture() ->driver_locked = true;
+	common->getPageCapture()->driver_locked = true;
 	return 0;
 }
 
@@ -930,14 +870,13 @@ int AVC::ResetHandler( raw1394handle_t handle, unsigned int generation )
 int AVC::isPhyIDValid( int phyID )
 {
 	int value = -1;
+	int currentNode, nodeCount;
+	rom1394_directory rom1394_dir;
+
 	pthread_mutex_lock( &avc_mutex );
 	if ( avc_handle != NULL )
 	{
-		int currentNode, nodeCount;
-		rom1394_directory rom1394_dir;
-
 		nodeCount = raw1394_get_nodecount( avc_handle );
-
 		if ( phyID >= 0 && phyID < nodeCount )
 		{
 			if ( rom1394_get_directory( avc_handle, phyID, &rom1394_dir ) >= 0 )
@@ -951,26 +890,49 @@ int AVC::isPhyIDValid( int phyID )
 			}
 		}
 
-		// look for a new AVC recorder
-		for ( currentNode = 0; value == -1 && currentNode < nodeCount; currentNode++ )
+		if ( value == -1 )
 		{
-			if ( rom1394_get_directory( avc_handle, currentNode, &rom1394_dir ) >= 0 )
+			raw1394_destroy_handle( avc_handle );
+			avc_handle = NULL;
+			port = -1;
+		}
+	}
+	for ( int p = 0; value == -1 && p < totalPorts; p++ )
+	{
+		if ( ( avc_handle = raw1394_new_handle_on_port( p ) ) )
+		{
+			port = p;
+			raw1394_set_bus_reset_handler( avc_handle, this->ResetHandler );
+
+			// look for a new AVC recorder
+			nodeCount = raw1394_get_nodecount( avc_handle );
+			for ( currentNode = 0; value == -1 && currentNode < nodeCount; currentNode++ )
 			{
-				if ( rom1394_get_node_type( &rom1394_dir ) == ROM1394_NODE_TYPE_AVC )
+				if ( rom1394_get_directory( avc_handle, currentNode, &rom1394_dir ) >= 0 )
 				{
-					if ( avc1394_check_subunit_type( avc_handle, currentNode, AVC1394_SUBUNIT_TYPE_VCR ) )
+					if ( rom1394_get_node_type( &rom1394_dir ) == ROM1394_NODE_TYPE_AVC )
 					{
-						// set Preferences to the newly found AVC node and return
-						octlet_t guid = rom1394_get_guid( avc_handle, currentNode );
-						snprintf( Preferences::getInstance().avcGUID, 64, "%08x%08x",
-					          ( quadlet_t ) ( guid >> 32 ), ( quadlet_t ) ( guid & 0xffffffff ) );
-						value = currentNode;
+						if ( avc1394_check_subunit_type( avc_handle, currentNode, AVC1394_SUBUNIT_TYPE_VCR ) )
+						{
+							// set Preferences to the newly found AVC node and return
+							octlet_t guid = rom1394_get_guid( avc_handle, currentNode );
+							snprintf( Preferences::getInstance().avcGUID, 64, "%08x%08x",
+								( quadlet_t ) ( guid >> 32 ), ( quadlet_t ) ( guid & 0xffffffff ) );
+							value = currentNode;
+						}
 					}
+					rom1394_free_directory( &rom1394_dir );
 				}
-				rom1394_free_directory( &rom1394_dir );
+			}
+			if ( value == -1 )
+			{
+				raw1394_destroy_handle( avc_handle );
+				avc_handle = NULL;
 			}
 		}
 	}
+	if ( value == -1 )
+		port = -1;
 	pthread_mutex_unlock( &avc_mutex );
 	return value;
 }
@@ -1194,49 +1156,353 @@ bool AVC::Timecode( int phyID, char* timecode )
 
 int AVC::getNodeId( const char *guid )
 {
+	int value = -1;
 	pthread_mutex_lock( &avc_mutex );
 	if ( avc_handle != NULL )
+	{
+		raw1394_destroy_handle( avc_handle );
+		avc_handle = NULL;
+	}
+	port = 0;
+	for ( int p = 0; value == -1 && p < totalPorts; p++ )
 	{
 		rom1394_directory rom1394_dir;
-		for ( int currentNode = 0; currentNode < raw1394_get_nodecount( avc_handle ); currentNode++ )
+		
+		if ( ( avc_handle = raw1394_new_handle_on_port( p ) ) )
 		{
-			if ( rom1394_get_directory( avc_handle, currentNode, &rom1394_dir ) >= 0 )
+			int nodeCount = raw1394_get_nodecount( avc_handle );
+
+			port = p;
+			raw1394_set_bus_reset_handler( avc_handle, this->ResetHandler );
+
+			for ( int currentNode = 0; value == -1 && currentNode < nodeCount; currentNode++ )
 			{
-				octlet_t currentGUID = rom1394_get_guid( avc_handle, currentNode );
-				char currentGUIDStr[ 65 ];
-				snprintf( currentGUIDStr, 64, "%08x%08x", ( quadlet_t ) ( currentGUID >> 32 ),
-			          ( quadlet_t ) ( currentGUID & 0xffffffff ) );
-				if ( strncmp( currentGUIDStr, guid, 64 ) == 0 )
+				if ( rom1394_get_directory( avc_handle, currentNode, &rom1394_dir ) >= 0 )
 				{
-					pthread_mutex_unlock( &avc_mutex );
-					return currentNode;
+					octlet_t currentGUID = rom1394_get_guid( avc_handle, currentNode );
+					char currentGUIDStr[ 65 ];
+					snprintf( currentGUIDStr, 64, "%08x%08x", ( quadlet_t ) ( currentGUID >> 32 ),
+						( quadlet_t ) ( currentGUID & 0xffffffff ) );
+					if ( strncmp( currentGUIDStr, guid, 64 ) == 0 )
+						value = currentNode;
+					rom1394_free_directory( &rom1394_dir );
 				}
-				rom1394_free_directory( &rom1394_dir );
+			}
+			if ( value == -1 )
+			{
+				raw1394_destroy_handle( avc_handle );
+				avc_handle = NULL;
 			}
 		}
 	}
+	if ( value == -1 )
+		port = -1;
 	pthread_mutex_unlock( &avc_mutex );
-	return -1;
+	return value;
 }
 
 
-dv1394Writer::dv1394Writer( string device, unsigned int channel, unsigned int nBuffers,
-                            unsigned int cip_n, unsigned int cip_d, unsigned int syt_offset ) : isInitialised( false )
-{
-	m_fd = -1;
-	m_deviceName = device;
-	m_channel = channel;
-	m_nBuffers = ( nBuffers > DV1394_MAX_FRAMES ) ? DV1394_MAX_FRAMES : nBuffers;
-	m_cip_n = cip_n;
-	m_cip_d = cip_d;
-	m_syt_offset = syt_offset;
 
-	m_fd = open( m_deviceName.c_str(), O_RDWR );
+IEEE1394Writer::IEEE1394Writer() :
+		m_isInitialised( false ), m_isRunning( false )
+{
+	pthread_mutex_init( &m_dequeMutex, NULL );
+	pthread_mutex_init( &m_conditionMutex, NULL );
+	pthread_cond_init( &m_condition, NULL );
+}
+
+IEEE1394Writer::~IEEE1394Writer()
+{
+	for ( int i = m_deque.size(); i > 0; --i )
+	{
+		Frame* frame = m_deque[ 0 ];
+		m_deque.pop_front();
+		GetFramePool()->DoneWithFrame( frame );
+	}
+	pthread_mutex_destroy( &m_conditionMutex );
+	pthread_cond_destroy( &m_condition );
+	pthread_mutex_destroy( &m_dequeMutex );
+}
+
+void IEEE1394Writer::StartThread()
+{
+	if ( m_isRunning )
+		return;
+	pthread_create( &m_thread, NULL, ThreadProxy, this );
+}
+
+void IEEE1394Writer::StopThread()
+{
+	if ( m_isRunning )
+	{
+		m_isRunning = false;
+		pthread_join( m_thread, NULL );
+	}
+}
+
+unsigned int IEEE1394Writer::WaitForAction( int seconds )
+{
+	pthread_mutex_lock( &m_dequeMutex );
+	unsigned int size = m_deque.size( );
+	pthread_mutex_unlock( &m_dequeMutex );
+
+	if ( size >= m_nBuffers )
+	{
+		pthread_mutex_lock( &m_conditionMutex );
+		if ( seconds == 0 )
+		{
+			pthread_cond_wait( &m_condition, &m_conditionMutex );
+			pthread_mutex_unlock( &m_conditionMutex );
+			pthread_mutex_lock( &m_dequeMutex );
+			size = m_deque.size( );
+		}
+		else
+		{
+			struct timeval tp;
+			struct timespec ts;
+			int result;
+
+			gettimeofday( &tp, NULL );
+			ts.tv_sec = tp.tv_sec + seconds;
+			ts.tv_nsec = tp.tv_usec * 1000;
+
+			result = pthread_cond_timedwait( &m_condition, &m_conditionMutex, &ts );
+			pthread_mutex_unlock( &m_conditionMutex );
+			pthread_mutex_lock( &m_dequeMutex );
+
+			size = m_deque.size();
+		}
+		pthread_mutex_unlock( &m_dequeMutex );
+	}
+
+	return size;
+}
+
+void IEEE1394Writer::TriggerAction( )
+{
+	pthread_mutex_lock( &m_conditionMutex );
+	pthread_cond_signal( &m_condition );
+	pthread_mutex_unlock( &m_conditionMutex );
+}
+
+void* IEEE1394Writer::ThreadProxy( void *arg )
+{
+	IEEE1394Writer* self = static_cast< IEEE1394Writer* >( arg );
+	return self->Thread();
+}
+
+
+#ifdef HAVE_IEC61883
+iec61883Writer::iec61883Writer( int port, unsigned int channel, unsigned int buffers )
+	: m_port( port ), m_handle( NULL ), m_iec61883dv( NULL ),
+	m_data( 0 ), m_index( -1 ), m_isSilent( false ), m_isReset( false )
+{
+	cerr << ">>> iec61883Writer::iec61883Writer port " << port << " channel " << channel << endl;
+	m_channel = channel;
+	m_nBuffers = buffers;
+}
+
+iec61883Writer::~iec61883Writer()
+{
+	// signal SendFrame to not accept incoming frame
+	TriggerAction();
+	m_isReset = true;
+
+	StopThread();
+
+	if ( m_iec61883dv )
+	{
+		iec61883_dv_close( m_iec61883dv );
+		raw1394_destroy_handle( m_handle );
+	}
+}
+
+
+int iec61883Writer::HandlerProxy( unsigned char *data, int n_dif_blocks,
+	unsigned int dropped, void *callback_data)
+{
+	if ( callback_data )
+	{
+		iec61883Writer* writer = static_cast< iec61883Writer* >( callback_data );
+		return writer->Handler( data, n_dif_blocks, dropped );
+	}
+	else
+	{
+		return -1;
+	}
+}
+
+
+int iec61883Writer::Handler( unsigned char *data, int n_dif_blocks, unsigned int dropped )
+{
+	// Initialize or reset current frame
+	if ( m_index <= 0 )
+	{
+		// Cancel thread by telling raw1394_iso to end
+		// It is important for reliability of next transmission in the same
+		// process that this occurs on a frame boundary!
+		if ( !m_isRunning || m_isReset )
+			return -1;
+
+		pthread_mutex_lock( &m_dequeMutex );
+		Frame* frame = m_deque[0];
+		pthread_mutex_unlock( &m_dequeMutex );
+
+		m_data = frame->data;
+		m_index = frame->GetFrameSize();
+	}
+
+	// Transfer the DV data to FireWire
+	memcpy( data, m_data, n_dif_blocks * 480 );
+
+	// Move data cursor
+	m_data += n_dif_blocks * 480;
+	m_index -= n_dif_blocks * 480;
+
+	// When we have sent all of the current frame
+	if ( m_index == 0 )
+	{
+		pthread_mutex_lock( &m_dequeMutex );
+		Frame* frame = m_deque[0];
+		if ( m_deque.size() > 1 )
+		{
+			// Discard the current frame only if more than one available
+			m_deque.pop_front();
+			GetFramePool()->DoneWithFrame( frame );
+
+			// Awaken a blocked parent thread
+			TriggerAction();
+
+			// frontmost frame has changed so we need to silence it if repeated
+			m_isSilent = false;
+		}
+		else if ( !m_isSilent )
+		{
+			AudioInfo info;
+			int16_t* silence[4];
+
+			// Make the repeated frame silent to avoid annoyance
+			for ( int i = 0; i < 4; i++ )
+				silence[i] = (int16_t*) calloc( 2 * DV_AUDIO_MAX_SAMPLES, sizeof( int16_t ) );
+			frame->GetAudioInfo( info );
+			info.channels = 2;
+			frame->EncodeAudio( info, silence );
+			for ( int i = 0; i < 4; i++ )
+				free( silence[i] );
+
+			// Do not need to silence this frame again
+			m_isSilent = true;
+		}
+		pthread_mutex_unlock( &m_dequeMutex );
+	}
+
+	return 0;
+}
+
+bool iec61883Writer::Open( bool isPAL )
+{
+	assert( m_handle == 0 );
+	m_isInitialised = false;
+	m_handle = raw1394_new_handle_on_port( m_port );
+	if ( m_handle != NULL )
+	{
+		raw1394_set_bus_reset_handler( m_handle, this->ResetHandlerProxy );
+		m_iec61883dv = iec61883_dv_xmit_init( m_handle, isPAL, HandlerProxy, this );
+		m_isInitialised = ( m_iec61883dv != NULL );
+		iec61883_dv_set_synch( m_iec61883dv, 1 );
+	}
+
+	return m_isInitialised;
+}
+
+void iec61883Writer::SendFrame( Frame &frame )
+{
+	if ( m_isReset )
+	{
+		StopThread();
+		if ( m_iec61883dv )
+		{
+ 			// iec61883_dv_close( m_iec61883dv ) hangs here
+			free( m_iec61883dv );
+			m_iec61883dv = NULL;
+		}
+		raw1394_destroy_handle( m_handle );
+		m_handle = NULL;
+		m_isInitialised = m_isReset = false;
+	}
+
+	// Block if too full
+	WaitForAction();
+
+	// Duplicate the frame
+	Frame *newFrame = GetFramePool()->GetFrame();
+	*newFrame = frame;
+
+	pthread_mutex_lock( &m_dequeMutex );
+	m_deque.push_back( newFrame );
+	pthread_mutex_unlock( &m_dequeMutex );
+
+	if ( !m_isInitialised && Open( frame.IsPAL() ) )
+	{
+		m_isInitialised = ( iec61883_dv_xmit_start( m_iec61883dv, m_channel ) == 0 );
+		StartThread();
+	}
+}
+
+void* iec61883Writer::Thread()
+{
+	// poll raw1394
+	struct pollfd pfd = {
+		fd: raw1394_get_fd( m_handle ),
+		events: POLLIN,
+		revents: 0
+	};
+	int result = 0;
+
+	m_isRunning = true;
+	while ( !m_isReset && result == 0 )
+	{
+		if ( poll( &pfd, 1, 100 ) > 0 && ( pfd.revents & POLLIN ) )
+			result = raw1394_loop_iterate( m_handle );
+	}
+
+	return NULL;
+}
+
+void iec61883Writer::ResetHandler( void )
+{
+	cerr << ">>> iec61883Writer::ResetHandler" << endl;
+	if ( m_isInitialised )
+		m_isReset = true;
+}
+
+int iec61883Writer::ResetHandlerProxy( raw1394handle_t handle, unsigned int generation )
+{
+	iec61883_dv_t dv = static_cast< iec61883_dv_t >( raw1394_get_userdata( handle ) );
+	iec61883Writer *self = static_cast< iec61883Writer* >( iec61883_dv_get_callback_data( dv ) );
+	raw1394_update_generation( handle, generation );
+	if ( self )
+		self->ResetHandler();
+	return 0;
+}
+#endif
+
+
+#ifdef HAVE_DV1394
+dv1394Writer::dv1394Writer( string device, unsigned int channel, unsigned int nBuffers,
+                            unsigned int cip_n, unsigned int cip_d, unsigned int syt_offset )
+	: m_channel( channel ), m_cip_n( cip_n ), m_cip_d( cip_d ), m_syt_offset( syt_offset ),
+	  m_isSilent( false )
+{
+	m_nBuffers = ( nBuffers > DV1394_MAX_FRAMES ) ? DV1394_MAX_FRAMES : nBuffers;
+	m_fd = open( device.c_str(), O_RDWR );
+	cerr << ">>> dv1394Writer::dv1394Writer " << device << " channel " << m_channel << " fd " << m_fd << endl;
 }
 
 
 dv1394Writer::~dv1394Writer()
 {
+	StopThread();
 	if ( m_fd != -1 )
 	{
 		close( m_fd );
@@ -1247,28 +1513,84 @@ dv1394Writer::~dv1394Writer()
 
 void dv1394Writer::SendFrame( Frame &frame )
 {
+	if ( m_fd < 0 )
+		return;
+
 	bool isPAL = frame.IsPAL();
 
-	if ( !isInitialised )
+	// Block if too full
+	WaitForAction();
+
+	// Duplicate the frame
+	Frame *newFrame = GetFramePool()->GetFrame();
+	*newFrame = frame;
+
+	pthread_mutex_lock( &m_dequeMutex );
+	m_deque.push_back( newFrame );
+	pthread_mutex_unlock( &m_dequeMutex );
+
+	if ( !m_isInitialised )
 	{
-		isInitialised = true;
+		m_isInitialised = true;
 
 		struct dv1394_init setup =
 		    {
-			api_version:
-						DV1394_API_VERSION,
-			channel:
-						m_channel,
-			n_frames:
-						m_nBuffers,
-			format:
-						( isPAL ? DV1394_PAL : DV1394_NTSC ),
+			api_version: DV1394_API_VERSION,
+			channel: m_channel,
+			n_frames: m_nBuffers,
+			format: ( isPAL ? DV1394_PAL : DV1394_NTSC ),
 			cip_n : m_cip_n,
 			cip_d : m_cip_d,
 			syt_offset : m_syt_offset
 		    };
 		ioctl( m_fd, DV1394_INIT, &setup );
+		StartThread();
+	}
+}
+
+void* dv1394Writer::Thread()
+{
+	m_isRunning = true;
+	while ( m_isRunning )
+	{
+		Frame* frame = m_deque[0];
+
+		if ( frame )
+			write( m_fd, frame->data, ( frame->IsPAL() ? DV1394_PAL_FRAME_SIZE : DV1394_NTSC_FRAME_SIZE ) );
+
+		pthread_mutex_lock( &m_dequeMutex );
+		if ( m_deque.size() > 1 )
+		{
+			// Discard the current frame only if more than one available
+			m_deque.pop_front();
+			GetFramePool()->DoneWithFrame( frame );
+
+			// Awaken a blocked parent thread
+			TriggerAction();
+
+			// frontmost frame has changed so we need to silence it if repeated
+			m_isSilent = false;
+		}
+		else if ( !m_isSilent )
+		{
+			AudioInfo info;
+			int16_t* silence[4];
+
+			// Make the repeated frame silent to avoid annoyance
+			for ( int i = 0; i < 4; i++ )
+				silence[i] = (int16_t*) calloc( 2 * DV_AUDIO_MAX_SAMPLES, sizeof( int16_t ) );
+			frame->GetAudioInfo( info );
+			info.channels = 2;
+			frame->EncodeAudio( info, silence );
+			for ( int i = 0; i < 4; i++ )
+				free( silence[i] );
+
+			// Do not need to silence this frame again
+			m_isSilent = true;
+		}
+		pthread_mutex_unlock( &m_dequeMutex );
 	}
 
-	write( m_fd, frame.data, ( isPAL ? DV1394_PAL_FRAME_SIZE : DV1394_NTSC_FRAME_SIZE ) );
+	return NULL;
 }
+#endif
