@@ -31,11 +31,13 @@
 #include "image_create.h"
 #include "page_magick.h"
 #include "message.h"
+#include "kino_extra.h"
 
 extern "C"
 {
 #include "support.h"
 	extern GladeXML* magick_glade;
+	extern void on_time_range_changed(GtkWidget *widget, gpointer user_data);
 }
 
 /*
@@ -58,12 +60,17 @@ protected:
 	GtkWidget *frame_counter;
 
 public:
-	ImageCreateColour()
+	ImageCreateColour() : frame_counter( 0 )
 	{
 		start.red = start.green = start.blue = 0;
 		frames = 25;
 	}
 
+	static void Repaint( void )
+	{
+		GetSelectedFramesForFX().Repaint();
+	}
+
 	char *GetDescription( ) const
 	{
 		return _( "Fixed Colour" );
@@ -95,8 +102,10 @@ public:
 
 		GtkObject *frame_counter_adj = gtk_adjustment_new( frames, 0, 999999, 1, 10, 10 );
 		frame_counter = gtk_spin_button_new( GTK_ADJUSTMENT( frame_counter_adj ), 1, 0 );
+		gtk_spin_button_set_value( GTK_SPIN_BUTTON( frame_counter ), frames );
 		gtk_widget_show( frame_counter );
 		gtk_widget_set_usize( frame_counter, 60, -2 );
+		g_signal_connect( G_OBJECT( frame_counter ), "value-changed", G_CALLBACK( on_time_range_changed ), NULL );
 
 		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, row, row + 1, ( GtkAttachOptions ) ( GTK_FILL ), ( GtkAttachOptions ) ( 0 ), 0, 0 );
 		gtk_table_attach( GTK_TABLE( table ), frame_counter, 1, 2, row, row + 1, ( GtkAttachOptions ) ( 0 ), ( GtkAttachOptions ) ( 0 ), 0, 0 );
@@ -104,8 +113,8 @@ public:
 
 	void InterpretFrameCollector( )
 	{
-		GtkEntry * entry = GTK_ENTRY( frame_counter );
-		frames = atoi( gtk_entry_get_text( entry ) );
+		if ( frame_counter )
+			frames = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( frame_counter ) );
 	}
 
 	void AttachWidgets( GtkBin *bin )
@@ -120,6 +129,7 @@ public:
 		picker = gtk_color_button_new();
 		gtk_color_button_set_color( GTK_COLOR_BUTTON( picker ), &start );
 		gtk_widget_show( picker );
+		g_signal_connect( G_OBJECT( picker ), "color-set", G_CALLBACK( Repaint ), 0 );
 
 		AddFrameCollector( table, 0 );
 
@@ -142,8 +152,11 @@ public:
 
 	void InterpretWidgets( GtkBin *bin )
 	{
-		InterpretFrameCollector( );
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( picker ), &start );
+		if ( bin->child != NULL )
+		{
+			InterpretFrameCollector( );
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( picker ), &start );
+		}
 	}
 
 	void CreateAudio( int16_t **buffer, short int *channels, int *frequency, int *samples )
@@ -196,7 +209,8 @@ public:
 
 	void InterpretWidgets( GtkBin *bin )
 	{
-		InterpretFrameCollector( );
+		if ( bin->child != NULL )
+			InterpretFrameCollector( );
 	}
 };
 
@@ -247,6 +261,7 @@ public:
 		start_picker = gtk_color_button_new();
 		gtk_color_button_set_color( GTK_COLOR_BUTTON( start_picker ), &start );
 		gtk_widget_show( start_picker );
+		g_signal_connect( G_OBJECT( start_picker ), "color-set", G_CALLBACK( Repaint ), 0 );
 
 		GtkWidget *end_label = gtk_label_new( _( "To:" ) );
 		gtk_widget_show( end_label );
@@ -254,6 +269,7 @@ public:
 		end_picker = gtk_color_button_new();
 		gtk_color_button_set_color( GTK_COLOR_BUTTON( end_picker ), &end );
 		gtk_widget_show( end_picker );
+		g_signal_connect( G_OBJECT( end_picker ), "color-set", G_CALLBACK( Repaint ), 0 );
 
 		AddFrameCollector( table, 0 );
 
@@ -279,9 +295,12 @@ public:
 
 	void InterpretWidgets( GtkBin *bin )
 	{
-		InterpretFrameCollector( );
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( start_picker ), &start );
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( end_picker ), &end );
+		if ( bin->child != NULL )
+		{
+			InterpretFrameCollector( );
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( start_picker ), &start );
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( end_picker ), &end );
+		}
 	}
 };
 
@@ -301,9 +320,21 @@ public:
 	{
 		window = glade_xml_get_widget( magick_glade, "image_create_gradiate" );
 		start_right.red = start_right.green = start_right.blue = 1 << 16;
-		gtk_color_button_set_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_start_right" ) ), &start_right );
+		GtkWidget* widget = lookup_widget( window, "colorpicker_start_right" );
+		gtk_color_button_set_color( GTK_COLOR_BUTTON( widget ), &start_right );
+		g_signal_connect( G_OBJECT( widget ), "color-set", G_CALLBACK( Repaint ), 0 );
+
 		end_right.red = end_right.green = end_right.blue = 1 << 16;
-		gtk_color_button_set_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_end_right" ) ), &end_right );
+		widget = lookup_widget( window, "colorpicker_end_right" );
+		gtk_color_button_set_color( GTK_COLOR_BUTTON( widget ), &end_right );
+		g_signal_connect( G_OBJECT( widget ), "color-set", G_CALLBACK( Repaint ), 0 );
+
+		widget = lookup_widget( window, "optionmenu_gradiate" );
+		g_signal_connect( G_OBJECT( widget ), "changed", G_CALLBACK( Repaint ), 0 );
+		widget = lookup_widget( window, "colorpicker_start_left" );
+		g_signal_connect( G_OBJECT( widget ), "color-set", G_CALLBACK( Repaint ), 0 );
+		widget = lookup_widget( window, "colorpicker_end_left" );
+		g_signal_connect( G_OBJECT( widget ), "color-set", G_CALLBACK( Repaint ), 0 );
 	}
 
 	virtual ~ImageCreateGradiate()
@@ -311,6 +342,11 @@ public:
 		gtk_widget_destroy( window );
 	}
 
+	static void Repaint( void )
+	{
+		GetSelectedFramesForFX().Repaint();
+	}
+
 	char *GetDescription( ) const
 	{
 		return _( "Gradiant" );
@@ -436,21 +472,24 @@ public:
 
 	void InterpretWidgets( GtkBin *bin )
 	{
-		GtkMenu * menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_gradiate" ) ) ) );
-		GtkWidget *active_item = gtk_menu_get_active( menu );
-		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
-
-		GtkEntry *entry = GTK_ENTRY( lookup_widget( window, "spinbutton_frames" ) );
-		frames = atoi( gtk_entry_get_text( entry ) );
-
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_start_left" ) ),
-		                           &start_left );
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_start_right" ) ),
-		                           &start_right );
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_end_left" ) ),
-		                           &end_left );
-		gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_end_right" ) ),
-		                           &end_right );
+		if ( bin->child != NULL )
+		{
+			GtkMenu * menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_gradiate" ) ) ) );
+			GtkWidget *active_item = gtk_menu_get_active( menu );
+			type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
+	
+			GtkEntry *entry = GTK_ENTRY( lookup_widget( window, "spinbutton_frames" ) );
+			frames = atoi( gtk_entry_get_text( entry ) );
+	
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_start_left" ) ),
+									&start_left );
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_start_right" ) ),
+									&start_right );
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_end_left" ) ),
+									&end_left );
+			gtk_color_button_get_color( GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker_end_right" ) ),
+									&end_right );
+		}
 	}
 
 	int GetNumberOfFrames( )
@@ -470,6 +509,9 @@ protected:
 	int count;
 
 public:
+	ImageCreateFromFile( ) : file_entry( 0 )
+	{ }
+
 	char *GetDescription( ) const
 	{
 		return _( "Create From File" );
@@ -479,6 +521,9 @@ public:
 	{
 		GError* gerror = NULL;
 		GdkPixbuf* still = NULL;
+
+		if ( strcmp( file, "" ) == 0 )
+			return;
 		
 		// Determine if this is an image sequence
 		if ( strchr( file, '%' ) != NULL )
@@ -556,6 +601,7 @@ public:
 		GtkWidget *hbox = gtk_hbox_new( FALSE, 6 );
 		file_entry = gtk_entry_new();
 		gtk_entry_set_text( GTK_ENTRY( file_entry ), file );
+		g_signal_connect( G_OBJECT( file_entry ), "changed", G_CALLBACK( Repaint ), 0 );
 		gtk_box_pack_start( GTK_BOX( hbox ), file_entry, TRUE, TRUE, 0 );
 		GtkWidget *button = gtk_button_new();
 		GtkWidget *image = gtk_image_new_from_stock( GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON );
@@ -597,17 +643,23 @@ public:
 
 	void InterpretWidgets( GtkBin *bin )
 	{
-		InterpretFrameCollector( );
-		strcpy( file, gtk_entry_get_text( GTK_ENTRY( file_entry ) ) );
-
-		if ( !strcmp( file, "" ) )
-			throw _( "No image file name specified - aborting." );
-		count = 0;
+		if ( bin->child != NULL )
+		{
+			InterpretFrameCollector( );
+			if ( file_entry )
+			{
+				strcpy( file, gtk_entry_get_text( GTK_ENTRY( file_entry ) ) );
+				count = 0;
+			}
+		}
 	}
 
 	GtkEntry *getEntry() const
 	{
-		return GTK_ENTRY( file_entry );
+		if ( file_entry )
+			return GTK_ENTRY( file_entry );
+		else
+			return NULL;
 	}
 };
 
