@@ -27,6 +27,7 @@
 class KinoAudioInput
 {
 public:
+	virtual ~KinoAudioInput() {}
 	virtual bool Open( char *input ) = 0;
 	virtual int GetChannels( ) = 0;
 	virtual int GetFrequency( ) = 0;
@@ -46,6 +47,7 @@ public:
 class KinoAudioPipe
 {
 public:
+	virtual ~KinoAudioPipe() {}
 	virtual bool OpenAudio( char *output, int channels, int frequency, int bytespersample ) = 0;
 	virtual bool OutputAudioFrame( int16_t *audio, int size ) = 0;
 	virtual bool CloseAudio( ) = 0;
@@ -65,6 +67,7 @@ public:
 class KinoVideoPipe
 {
 public:
+	virtual ~KinoVideoPipe() {}
 	virtual bool OpenVideoPipe( char *pipe, int width, int height, bool interlaced = true ) = 0;
 	virtual bool OutputVideoFrame( uint8_t *frame, int size ) = 0;
 	virtual bool CloseVideo( ) = 0;
