@@ -67,6 +67,17 @@ extern "C"
 #include <pthread.h>
 }
 
+GtkWindow * KinoCommon::getWidgetWindow(GtkWidget *widget)
+{
+	if(widget)
+		if(GTK_IS_WINDOW(widget))
+			return GTK_WINDOW(widget);
+		else
+			return getWidgetWindow(widget->parent);
+	else
+		return NULL;
+}
+
 /** Constructor for KinoCommon - initialises all GUI widgets.
 
   	\param widget top level widget for the main window
@@ -547,19 +558,27 @@ std::string KinoCommon::importFile( const char *filename )
 {
 	std::string importedFile( filename );
 
-	if ( modal_confirm( GTK_STOCK_OK, NULL, 
+	if ( modal_confirm( GTK_STOCK_OK, NULL,
 		_("This is not a DV file. Do you want to import it?") )
 		== GTK_RESPONSE_ACCEPT )
 	{
 		char * args[ 7 ];
 		int pid, status = -1;
 		GError *gerror = NULL;
-		gfloat val = 0.0;
 		gboolean visible = TRUE;
 		GtkWidget *dialog = lookup_widget( widget, "import_window" );
 		GtkProgressBar *progressBar = GTK_PROGRESS_BAR( lookup_widget( widget, "progressbar_import" ) );
+		char *kinoHome = getenv("KINO_HOME");
+		string homeFile;
+		int testHomeFile = 0;
 
-		gtk_progress_bar_update( progressBar, val );
+		if ( kinoHome )
+			homeFile = string( kinoHome ) + string( "/import/media.sh" );
+		else
+			homeFile = string( getenv( "HOME" ) ) + string( "/kino/import/media.sh" );
+
+		gtk_window_set_transient_for( GTK_WINDOW( dialog ), GTK_WINDOW( getWidgetWindow( getWidget() ) ) );
+		gtk_progress_bar_pulse( progressBar );
 		gtk_widget_show_all( dialog );
 
 		// Check if media is loaded
@@ -604,10 +623,20 @@ std::string KinoCommon::importFile( const char *filename )
 			GetFramePool()->DoneWithFrame( &frame );
 		}
 		importedFile += ".dv";
+		
 		args[ 0 ] = DATADIR "/kino/scripts/import/media.sh";
 		args[ 1 ] = const_cast<char*>( filename );
 		args[ 2 ] = const_cast<char*>( importedFile.c_str() );
 		args[ 6 ] = NULL;
+
+		// Allow a script in home directory to override
+		testHomeFile = open( homeFile.c_str(), O_RDONLY );
+		if ( testHomeFile > -1 )
+		{
+			close( testHomeFile );
+			args[ 0 ] = const_cast< char* >( homeFile.c_str() );
+		}
+
 		g_spawn_async_with_pipes( ".", args, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
 			NULL, NULL, &pid, NULL, NULL, NULL, &gerror );
 		
@@ -618,11 +647,8 @@ std::string KinoCommon::importFile( const char *filename )
 			while ( gtk_events_pending() )
 				gtk_main_iteration();
 			nanosleep( &ts, NULL );
-			gtk_progress_bar_update( progressBar, val > 1.0 ? 1.0 : val );
-			// We do not know how long it will take, just show something is
-			// happening by incrementing progress
-			val = ( val >= 1.0 ) ? 0.0 : ( val + 0.001 );
- 			g_object_get( G_OBJECT( dialog ), "visible", &visible, NULL );
+			gtk_progress_bar_pulse( progressBar );
+ 			g_object_get( G_OBJECT( dialog ), "visible", &visible, static_cast<gchar*>( NULL ) );
 			waitpid( pid, &status, WNOHANG );
 		}
 		gtk_widget_hide( dialog );
@@ -970,18 +996,20 @@ void update_preview_cb (GtkFileChooser *file_chooser, gpointer data)
 	\return a string inidicating the selected file (an empty string denotes no selection)
 */
 
-char *KinoCommon::getFileToOpen( char *title, bool isDVFile )
+char *KinoCommon::getFileToOpen( char *title, bool isDVFile, GtkWidget *widget )
 {
 	GtkWidget *dialog;
 	GtkDrawingArea *preview;
 	GtkFileFilter *filter;
 
 	dialog = gtk_file_chooser_dialog_new( title,
-				NULL,
+				getWidgetWindow(widget),
 				GTK_FILE_CHOOSER_ACTION_OPEN,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-				NULL );
+				static_cast<gchar*>( NULL ) );
+	gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
+
 	if ( isDVFile )
 	{
 		preview = ( GtkDrawingArea* )gtk_drawing_area_new();
@@ -997,12 +1025,13 @@ char *KinoCommon::getFileToOpen( char *title, bool isDVFile )
 		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(dialog), filter );
 	
 		filter = gtk_file_filter_new();
-		gtk_file_filter_set_name( filter, "SMIL Files (*.smil)" );
+		gtk_file_filter_set_name( filter, "SMIL Files (*.smil, *.kino)" );
 		gtk_file_filter_add_pattern( filter, "*.smil" );
+		gtk_file_filter_add_pattern( filter, "*.kino" );
 		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(dialog), filter );
 	
 		filter = gtk_file_filter_new();
-		gtk_file_filter_set_name( filter, "Digital Video Files (*.dv, *.dif)" );
+		gtk_file_filter_set_name( filter, "Raw DV Files (*.dv, *.dif)" );
 		gtk_file_filter_add_pattern( filter, "*.dv" );
 		gtk_file_filter_add_pattern( filter, "*.dif" );
 		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(dialog), filter );
@@ -1025,20 +1054,21 @@ char *KinoCommon::getFileToOpen( char *title, bool isDVFile )
 	{
 		string directory = getPlayList() ->GetProjectDirectory( );
 		if ( directory != "" )
-			gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( directory + "/." ).c_str() );
+			gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( directory + "/" ).c_str() );
 		last_directory = directory;
 	}
 	else
 	{
-		gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( last_directory + "/." ).c_str() );
+		gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( last_directory + "/" ).c_str() );
 	}
 	if (gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT) {
-		const gchar *filename = gtk_file_chooser_get_filename(
+		gchar *filename = gtk_file_chooser_get_filename(
 						GTK_FILE_CHOOSER( dialog ));
 		if ( filename )
 		{
 			strncpy( tempFileName, filename, sizeof(tempFileName) );
 			last_directory = directory_utils::get_directory_from_file( filename );
+			g_free( filename );
 		}
 		else
 			strcpy( tempFileName, "" );
@@ -1049,27 +1079,28 @@ char *KinoCommon::getFileToOpen( char *title, bool isDVFile )
 	return tempFileName;
 }
 
-char *KinoCommon::getFileToSave( char *title )
+char *KinoCommon::getFileToSave( char *title, GtkWidget *widget )
 {
 	GtkWidget *dialog;
 
-	dialog = gtk_file_chooser_dialog_new(title,
-					NULL,
+	dialog = gtk_file_chooser_dialog_new( title,
+					getWidgetWindow(widget),
 					GTK_FILE_CHOOSER_ACTION_SAVE,
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-					NULL);
+					static_cast<gchar*>( NULL ) );
+	gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
 	gtk_window_set_modal( GTK_WINDOW( dialog ), TRUE );
 	if ( last_directory == "" )
 	{
 		string directory = getPlayList() ->GetProjectDirectory( );
 		if ( directory != "" )
-			gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( directory + "/." ).c_str() );
+			gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( directory + "/" ).c_str() );
 		last_directory = directory;
 	}
 	else
 	{
-		gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( last_directory + "/." ).c_str() );
+		gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), ( char * ) ( last_directory + "/" ).c_str() );
 	}
 	if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 		char *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
@@ -1121,6 +1152,7 @@ char *KinoCommon::getFileToOpen( char *title, bool isDVFile )
 	}
 	else
 		strcpy( tempFileName, "" );
+	g_free( filename );
 	gtk_widget_destroy( dialog );
 	return tempFileName;
 }
@@ -1444,9 +1476,9 @@ void KinoCommon::saveFrame( int abs_frame_num, char *file )
 		const std::string FileName( file );
 		const std::string suffix( FileName.begin() + FileName.rfind( "." ), FileName.end() );
 		if ( suffix == ".png" )
-			gdk_pixbuf_save( im, file, "png", &gerror, NULL );
+			gdk_pixbuf_save( im, file, "png", &gerror, static_cast<gchar*>( NULL ) );
 		else
-			gdk_pixbuf_save( im, file, "jpeg", &gerror, "quality", "80", NULL );
+			gdk_pixbuf_save( im, file, "jpeg", &gerror, "quality", "80", static_cast<gchar*>( NULL ) );
 		if ( gerror != NULL )
 		{
 			modal_message( gerror->message );
@@ -1490,6 +1522,10 @@ int KinoCommon::moveToFrame( )
 		getPageTrim() ->movedToFrame( getPageTrim() ->getPosition() );
 		return getPageTrim() ->getPosition();
 	}
+	else if ( currentPage == PAGE_MAGICK )
+	{
+		return moveToFrame( g_currentFrame - playlist.FindStartOfScene( g_currentFrame ) );
+	}
 	return moveToFrame( g_currentFrame );
 }
 
@@ -1508,6 +1544,11 @@ int KinoCommon::moveToFrame( int frame )
 		getPageTrim() ->movedToFrame( frame );
 		return frame;
 	}
+	if ( currentPage == PAGE_MAGICK )
+	{
+		getPageMagick() ->movedToFrame( frame );
+		return frame;
+	}
 
 	if ( frame >= 0 && frame < getPlayList() ->GetNumFrames() )
 		g_currentFrame = frame;
@@ -1563,7 +1604,7 @@ void KinoCommon::showFrameMoreInfo( Frame &frame, FileHandler *media )
 	if ( showMoreInfo )
 	{
 		GtkLabel *label = GTK_LABEL( lookup_widget( widget, "label_properties" ) );
-		char s[ 2048 ];
+		char* s = new char[ 2048 ];
 
 		if ( media != NULL )
 		{
@@ -1580,12 +1621,12 @@ void KinoCommon::showFrameMoreInfo( Frame &frame, FileHandler *media )
 			else if ( ext == ".avi" )
 			{
 				AVIHandler * avi = dynamic_cast< AVIHandler* >( media );
-				format = ( avi->GetFormat() == AVI_DV1_FORMAT ) ? "AVI, FOURCC dvsd, Type 1" : "AVI, FOURCC dvsd, Type 2";
+				format = ( avi->GetFormat() == AVI_DV1_FORMAT ) ? "AVI, Type 1" : "AVI, Type 2";
 				if ( avi->GetOpenDML() )
 					format += ", OpenDML";
 			}
 			else if ( ext == ".mov" )
-				format = "Quicktime, FOURCC dvsd";
+				format = "Quicktime";
 			else
 				format = "unknown";
 
@@ -1606,6 +1647,7 @@ void KinoCommon::showFrameMoreInfo( Frame &frame, FileHandler *media )
 			sprintf( s, "\n\n\n" );
 
 		gtk_label_set_text( label, s );
+		delete[] s;
 	}
 }
 
@@ -1622,7 +1664,8 @@ void KinoCommon::start()
 	activateWidgets();
 
 	toggleComponents( getComponentState(), false );
-	toggleComponents( VIDEO_STOP, true );
+	if ( currentPage != PAGE_MAGICK )
+		toggleComponents( VIDEO_STOP, true );
 	commitComponentState();
 }
 
