@@ -156,6 +156,9 @@ private:
 	SMIL::MediaClippingTime time;
 
 public:
+
+    static GtkWindow * getWidgetWindow( GtkWidget *widget );
+
 	KinoCommon( GtkWidget *widget );
 	~KinoCommon();
 
@@ -246,8 +249,16 @@ public:
 	void commitComponentState( component_enum = ( component_enum ) 0 );
 
 	// File Dialogs
-	char *getFileToOpen( char *, bool isDVFile = true );
-	char *getFileToSave( char * );
+	char *getFileToOpen( char *title, bool isDVFile, GtkWidget *widget );
+	char *getFileToSave( char *title, GtkWidget *widget );
+	char *getFileToOpen( char *title, bool isDVFile = true )
+	{
+		return getFileToOpen( title, isDVFile, getWidget() );
+	}
+	char *getFileToSave( char *title )
+	{
+		return getFileToSave( title, getWidget() );
+	}
 
 	// misc
 	void setPreviewSize( float factor, bool noWarning = false );
