@@ -24,12 +24,38 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include <stdarg.h>
+#include "kino_common.h"
+#include "message.h"
 
 extern "C"
 {
 	extern GtkWidget *main_window;
 
-	void modal_message( const char * message )
+	void modal_message ( const char * message )
+	{
+		return modal_message_with_parent( message, common->getWidget());
+	}
+
+	char *modal_prompt( const char * message )
+	{
+		return modal_prompt_with_parent( message, common->getWidget());
+	}
+
+	int modal_confirm( const char * affirm_label, const char *close_label, const char *message, ... )
+	{
+		gchar* msg;
+		va_list args;
+		int rv=0;
+
+		va_start( args, message );
+		msg = g_strdup_vprintf( message, args );
+		va_end( args );
+		rv=modal_confirm_with_parent( affirm_label, close_label, common->getWidget(), msg, NULL);
+		g_free( msg );
+		return rv;
+	}
+
+	void modal_message_with_parent( const char * message, GtkWidget *parent )
 	{
 
 		GtkWidget * dialog = gtk_message_dialog_new(
@@ -39,11 +65,15 @@ extern "C"
 		                         GTK_BUTTONS_OK,
 		                         message );
 		gtk_window_set_resizable( GTK_WINDOW( dialog ), FALSE );
+
+		if(parent)
+			gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(KinoCommon::getWidgetWindow(parent)));
+
 		gtk_dialog_run( GTK_DIALOG( dialog ) );
 		gtk_widget_destroy( dialog );
 	}
 
-	char *modal_prompt( const char * message )
+	char *modal_prompt_with_parent( const char * message, GtkWidget *parent )
 	{
 		GtkWidget *dialog, *hbox1, *hbox2, *label, *entry, *icon;
 		char *response = NULL;
@@ -58,6 +88,7 @@ extern "C"
                                          GTK_STOCK_OK,
                                          GTK_RESPONSE_ACCEPT,
                                          NULL);
+	    gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_REJECT, -1);
 		gtk_dialog_set_default_response( GTK_DIALOG( dialog ), GTK_RESPONSE_ACCEPT );
 		gtk_dialog_set_has_separator( GTK_DIALOG( dialog ), FALSE );
 		gtk_window_set_resizable( GTK_WINDOW( dialog ), FALSE );
@@ -79,6 +110,10 @@ extern "C"
 		gtk_container_add( GTK_CONTAINER( hbox2 ), entry );
 		gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->vbox ), hbox1 );
 		gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->vbox ), hbox2 );
+
+		if(parent)
+			gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(KinoCommon::getWidgetWindow(parent)));
+
 		gtk_widget_show_all( dialog );
 		
 		if ( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT )
@@ -88,7 +123,7 @@ extern "C"
 		return response;
 	}
 
-int modal_confirm( const char *affirm_label, const char *close_label, const char *message, ... )
+int modal_confirm_with_parent( const char *affirm_label, const char *close_label, GtkWidget *parent, const char *message, ... )
 {
 	int result = GTK_RESPONSE_NONE;
 	va_list args;
@@ -110,13 +145,21 @@ int modal_confirm( const char *affirm_label, const char *close_label, const char
 		gtk_dialog_add_button( GTK_DIALOG( dialog ), close_label, GTK_RESPONSE_CLOSE );
 	gtk_dialog_add_button( GTK_DIALOG( dialog ), GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT );
 	gtk_dialog_add_button( GTK_DIALOG( dialog ), affirm_label, GTK_RESPONSE_ACCEPT );
+	if ( close_label != NULL )
+	    gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CLOSE, GTK_RESPONSE_REJECT, -1);
+	else
+	    gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_REJECT, -1);
 	gtk_dialog_set_default_response( GTK_DIALOG( dialog ), GTK_RESPONSE_ACCEPT );
-	gtk_window_set_title( GTK_WINDOW( dialog ), "" );
+	gtk_window_set_title( GTK_WINDOW( dialog ), _("Question") );
 	gtk_window_set_resizable( GTK_WINDOW( dialog ), FALSE );
 
+	if(parent)
+		gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(KinoCommon::getWidgetWindow(parent)));
+
 	result = gtk_dialog_run( GTK_DIALOG( dialog ) );
 	gtk_widget_destroy( dialog );
 	
+	g_free(	msg );
 	return result;
 }
 
