@@ -149,16 +149,16 @@ void PageCapture::start()
 	gtk_label_set_text( label, "" );
 
 	// Create the reader object
-	if ( Preferences::getInstance().dvCaptureDriver == 0 )
-		reader = new raw1394Reader( Preferences::getInstance().interface,
-		                            Preferences::getInstance().channel,
-		                            Preferences::getInstance().dvCaptureBuffers );
-	else
-		reader = new dv1394Reader( Preferences::getInstance().channel,
-		                           Preferences::getInstance().dvCaptureBuffers );
+#ifdef HAVE_IEC61883
+	reader = new iec61883Reader( Preferences::getInstance().channel,
+	                             Preferences::getInstance().dvCaptureBuffers );
+#else
+	reader = new dv1394Reader( Preferences::getInstance().channel,
+	                           Preferences::getInstance().dvCaptureBuffers );
+#endif
 
 	// Create the AV/C object
-	avc = new AVC( Preferences::getInstance().interface );
+	avc = new AVC();
 
 	// Determine the capture directory and file - rules are:
 	// 1) if a full path is specified in the preferences, then use that
@@ -217,26 +217,25 @@ void PageCapture::CheckDevices( )
 	}
 	else if ( driver_available == false )
 	{
+		// Check if the id of the device is valid
+		Preferences::getInstance( ).phyID = avc->getNodeId( Preferences::getInstance( ).avcGUID );
+		Preferences::getInstance( ).phyID = avc->isPhyIDValid( Preferences::getInstance( ).phyID );
+		check_phyid = Preferences::getInstance( ).phyID < 0;
+
 		// Start the reader thread
 		//std::cerr << "Starting thread" << std::endl;
-		if ( reader->StartThread() == true )
+		if ( reader->StartThread( avc->getPort() ) )
 		{
 			// The drivers are installed
 			driver_available = true;
 			avc_enabled = false;
-
-			// Check if the id of the device is valid
-			Preferences::getInstance( ).phyID = avc->isPhyIDValid( avc->getNodeId( Preferences::getInstance( ).avcGUID ) );
-			Preferences::getInstance( ).phyID = avc->getNodeId( Preferences::getInstance( ).avcGUID );
-			check_phyid = Preferences::getInstance( ).phyID < 0;
 		}
 		TriggerAction( );
 	}
 	else if ( check_phyid )
 	{
 		// Check that we're still valid.
-		Preferences::getInstance( ).phyID = avc->isPhyIDValid( avc->getNodeId( Preferences::getInstance( ).avcGUID ) );
-		Preferences::getInstance( ).phyID = avc->getNodeId( Preferences::getInstance( ).avcGUID );
+		Preferences::getInstance( ).phyID = avc->isPhyIDValid( Preferences::getInstance( ).phyID );
 		check_phyid = Preferences::getInstance( ).phyID < 0;
 		avc_enabled = false;
 		TriggerAction( );
