@@ -64,7 +64,7 @@ extern "C"
 #define PLAYBACK_FRAMES 50
 
 	static Frame *frameContent[ PLAYBACK_FRAMES ];
-	dv1394Writer *dv1394;
+	IEEE1394Writer *writer1394 = NULL;
 	static void TriggerQueueAction( );
 
 	static gboolean doScrub = FALSE;
@@ -116,7 +116,6 @@ PageEditor::PageEditor( KinoCommon *common )
 	this->positionLabelTotal = GTK_LABEL( lookup_widget( common->getWidget(), "position_label_total" ) );
 	this->lastFrameShown = -1;
 	this->g_copiedPlayList = new PlayList();
-	dv1394 = NULL;
 
 	scrubAdjustment = GTK_ADJUSTMENT( gtk_adjustment_new( 0, 0, 0, 1, 10, 0 ) );
 	g_signal_connect( G_OBJECT( scrubAdjustment ), "value_changed",
@@ -170,16 +169,29 @@ void PageEditor::start()
 		frameContent[ i ] = GetFramePool( ) ->GetFrame( );
 
 	this->displayer = new FrameDisplayer();
-	if ( Preferences::getInstance().dvExportDevice > 0
-	        && Preferences::getInstance().dv1394Preview )
-		dv1394 = new dv1394Writer(
-		             Preferences::getInstance().dvExportDevice,
-		             Preferences::getInstance().channel,
-		             Preferences::getInstance().dvExportBuffers,
-		             Preferences::getInstance().cip_n,
-		             Preferences::getInstance().cip_d,
-		             Preferences::getInstance().syt_offset
-		         );
+	if ( Preferences::getInstance().dv1394Preview )
+	{
+#ifdef HAVE_IEC61883
+		AVC avc;
+		Preferences::getInstance( ).phyID = avc.getNodeId( Preferences::getInstance( ).avcGUID );
+		Preferences::getInstance( ).phyID = avc.isPhyIDValid( Preferences::getInstance( ).phyID );
+		if ( Preferences::getInstance( ).phyID >= 0 && !writer1394 )
+			writer1394 = new iec61883Writer(
+			             avc.getPort(),
+			             Preferences::getInstance().channel,
+			             2 );
+#else
+		if ( Preferences::getInstance().dvExportDevice > 0 && !writer1394 )
+			writer1394 = new dv1394Writer(
+			             Preferences::getInstance().dvExportDevice,
+			             Preferences::getInstance().channel,
+			             2,
+			             Preferences::getInstance().cip_n,
+			             Preferences::getInstance().cip_d,
+			             Preferences::getInstance().syt_offset );
+#endif
+	}
+
 	this->lastFrameShown = common->g_currentFrame;
 
 	gtk_widget_grab_focus( GTK_WIDGET( frameArea ) );
@@ -219,11 +231,8 @@ void PageEditor::clean()
 	cerr << ">> Leaving Editor" << endl;
 	stopNavigator();
 	delete this->displayer;
-	if ( dv1394 != NULL )
-	{
-		delete dv1394;
-		dv1394 = NULL;
-	}
+	delete writer1394;
+	writer1394 = NULL;
 	for ( int i = 0; i < PLAYBACK_FRAMES; i ++ )
 		GetFramePool( ) ->DoneWithFrame( frameContent[ i ] );
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( lookup_widget( common->getWidget(), "menuitem_editor" ) ), FALSE );
@@ -298,8 +307,8 @@ void PageEditor::showFrame( int i, gboolean no_audio )
 		Frame *frame = GetFramePool( ) ->GetFrame( );
 		common->getPlayList() ->GetFrame( i, *frame );
 		common->showFrameInfo( i );
-		if ( dv1394 != NULL )
-			dv1394->SendFrame( *frame ) ;
+		if ( writer1394 != NULL )
+			writer1394->SendFrame( *frame ) ;
 		getFrameDisplayer() ->Put( *frame, GTK_WIDGET( frameArea ), no_audio );
 		lastFrameShown = i;
 		GetFramePool( ) ->DoneWithFrame( frame );
@@ -899,9 +908,12 @@ gboolean PageEditor::processCommand( char *command )
 
 	else if ( ( strcmp( cmd, "o" ) == 0 ) || strcmp( cmd, "d$" ) == 0 )
 	{
+		start = common->g_currentFrame;
+		if ( strcmp( cmd, "o" ) == 0 )
+			++start;
 		end = common->getPlayList() ->FindEndOfScene( common->g_currentFrame );
-		CopyFrames( common->g_currentFrame, end );
-		DeleteFrames( common->g_currentFrame, end );
+		CopyFrames( start, end );
+		DeleteFrames( start, end );
 		common->moveToFrame( );
 		common->keyboardFeedback( cmd, _( "Cut to end of scene" ) );
 		DrawBar( common->g_currentFrame );
@@ -1276,13 +1288,16 @@ void PageEditor::showFrame( int position, Frame& frame )
 
 void PageEditor::windowMoved()
 {
-	if ( common->getPlayList() ->GetNumFrames() )
+	if ( g_nav_ctl.active == FALSE )
 	{
-		showFrame( common->g_currentFrame, TRUE );
-	}
-	else
-	{
-		common->loadSplash( frameArea );
+		if ( common->getPlayList() ->GetNumFrames() )
+		{
+			showFrame( common->g_currentFrame, TRUE );
+		}
+		else
+		{
+			common->loadSplash( frameArea );
+		}
 	}
 }
 
@@ -1561,8 +1576,8 @@ extern "C"
 				
 				TriggerAudioAction( );
 
-				if ( dv1394 != NULL )
-					dv1394->SendFrame( *frameContent[ tail ] );
+				if ( writer1394 != NULL )
+					writer1394->SendFrame( *frameContent[ tail ] );
 					
 				// We have a frame, so start playing
 				playing = true;
