@@ -25,6 +25,7 @@
 #include <iostream>
 using std::cerr;
 using std::endl;
+#include <string>
 
 #include <gtk/gtk.h>
 #include <stdio.h>
@@ -38,6 +39,7 @@ using std::endl;
 #include "page_editor.h"
 #include "message.h"
 
+
 /** Constructor for page.
 
 \param common	common object to which this page belongs
@@ -152,14 +154,9 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 		type = PIPE_AUDIO_WAV;
 	}
 
-	// Create the resampler and output objects
-	AudioResample<int16_ne_t,int16_le_t> *resampler = AudioResampleFactory<int16_ne_t,int16_le_t>::createAudioResample(
-	                               AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, resampleRate );
-	KinoAudioPipe *output = KinoAudioFactory::CreateAudioPipe( type );
-
 	// Iterate through each scene (opening and closing audio pipes as requested)
+	KinoAudioPipe *outputPipe = KinoAudioFactory::CreateAudioPipe( type );
 	int scene = 0;
-
 	for ( int sceneBegin = begin; sceneBegin <= end && exportPage->isExporting;
 	        scene ++ )
 	{
@@ -171,7 +168,7 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 		if ( splitScene )
 		{
 			sprintf( sceneString, "%03d", scene );
-			sceneEnd = playlist->FindEndOfScene( sceneBegin ) + 1;
+			sceneEnd = playlist->FindEndOfScene( sceneBegin );
 			if ( sceneEnd > end )
 				sceneEnd = end;
 		}
@@ -184,32 +181,54 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 		full = g_strdup_printf( outputtemplate, sceneString );
 
 		// output this scene
-		bool active = output->OpenAudio( full, channels, resampleRate, 2 );
+		bool active = outputPipe->OpenAudio( full, channels, resampleRate, 2 );
 		g_free( full );
 		
 		if ( active )
 		{
-			for ( int i = sceneBegin; i <= sceneEnd && exportPage->isExporting;
-			        i += every )
+			// Call this before hand to initialise the counters
+			// calculateAdjustedRate will accumulate time as paused time to prevent
+			// distorting the estimate.
+			innerLoopUpdate( sceneBegin, begin, end, every );
+
+			// Determine correct amount of audio for duration
+			double adjustedRate = calculateAdjustedRate( playlist, resampleRate, sceneBegin, sceneEnd, every );
+			if ( ! adjustedRate )
+				status = EXPORT_RESULT_ABORT;
+
+			// Setup a resampler
+			AsyncAudioResample<int16_ne_t,int16_le_t>* resampler = new AsyncAudioResample<int16_ne_t,int16_le_t>(
+				AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, playlist, resampleRate, sceneBegin, sceneEnd, every );
+			if ( resampler->IsError() )
+			{
+				std::cerr << ">>> Resampler error: " << resampler->GetError() << std::endl;
+				exportPage->isExporting = false;
+				status = EXPORT_RESULT_FAILURE;
+				break;
+			}
+
+			// Now run through the frames in this scene
+			for ( int i = sceneBegin, j = 0; i <= sceneEnd && exportPage->isExporting;
+			        i += every, j++ )
 			{
-				/* Call innerLoopUpdate */
 				innerLoopUpdate( i, begin, end, every );
 
-				playlist->GetFrame( i, frame );
-				resampler->Resample( frame );
-				if ( !output->OutputAudioFrame( reinterpret_cast<int16_t *>(resampler->output), resampler->size ) )
+				int requestedSamples = frame.CalculateNumberSamples( resampleRate, j );
+				int nsamples = resampler->Process( adjustedRate, requestedSamples );
+				if ( nsamples > 0 && !outputPipe->OutputAudioFrame( resampler->GetOutput(), nsamples * channels * sizeof(int16_t) ) )
 				{
 					modal_message( _( "Error during audio export - aborting." ) );
 					status = EXPORT_RESULT_FAILURE;
 					exportPage->isExporting = false;
 				}
-				/* Update progressbar ... */
-				exportPage->updateProgress( ( gfloat ) ( i + every - begin )
-				                            / ( gfloat ) ( end + 1 - begin ) );
 			}
 			if ( status == EXPORT_RESULT_SUCCESS && !exportPage->isExporting )
 				status = EXPORT_RESULT_ABORT;
-			output->CloseAudio();
+			
+			// Cleanup
+			outputPipe->CloseAudio();
+			delete resampler;
+			resampler = 0;
 		}
 		else
 		{
@@ -224,8 +243,7 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 
 	g_free( file );
 	g_free( outputtemplate );
-	delete resampler;
-	delete output;
+	delete outputPipe;
 
 	GetFramePool()->DoneWithFrame( &frame );
 	
