@@ -21,6 +21,7 @@
 #include <config.h>
 #endif
 
+#include <algorithm>
 #include <fstream>
 #include <iostream>
 #include <iomanip>
@@ -33,6 +34,8 @@ using std::endl;
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <dirent.h>
 
 #include "page_export_mjpeg.h"
 #include "kino_av_pipe.h"
@@ -41,6 +44,74 @@ using std::endl;
 #include "frame.h"
 #include "page_editor.h"
 #include "message.h"
+#include "rwpipe.h"
+
+
+class DVDTool : protected RWPipe
+{
+public:
+	string m_command;
+	string m_description;
+	bool m_active;
+
+	DVDTool( string command ) :
+			m_command( command ),
+			m_description( "" ),
+			m_active( false )
+	{
+		load( command );
+	}
+
+	void load( string command )
+	{
+		if ( run( command ) )
+		{
+			char temp[ 10240 ];
+			while ( readLine( temp, 10240 ) > 0 )
+			{
+				char * ptr = strchr( temp, ':' );
+				if ( ptr != NULL )
+				{
+					*ptr = '\0';
+					ptr += 2;
+
+					if ( !strcmp( temp, "Title" ) )
+						m_description = ptr;
+					else if ( !strcmp( temp, "Status" ) )
+						m_active = !strcmp( ptr, "Active" );
+					else
+						fprintf( stderr, "Unrecognised %s: %s\n", temp, ptr );
+				}
+			}
+			stop( );
+		}
+	}
+
+	bool execute( const string& dvdauthorFile, const string& outputDirectory )
+	{
+		gchar *command;
+		bool result;
+
+		// Construct the command
+		command = g_strdup_printf( "%s \"%s\" \"%s\"", m_command.c_str( ),
+		         dvdauthorFile.c_str( ), outputDirectory.c_str() );
+
+		result = system( command );
+		g_free( command );
+		return result;
+	}
+
+	void close( )
+	{
+		stop( );
+	}
+
+	static bool compare( const DVDTool* x, const DVDTool* y )
+	{
+		return ( x->m_description < y->m_description );
+	}
+};
+
 
 /** Constructor for page.
 
@@ -55,6 +126,21 @@ ExportMJPEG::ExportMJPEG( PageExport *_exportPage, KinoCommon *_common ) :
 {
 	cerr << "> Creating ExportMJPEG Page" << endl;
 
+	// Load the extensible dvdauthor tools
+	loadTools( DATADIR "/kino/scripts/dvdauthor" );
+	string alternate_dir = string( getenv( "HOME" ) ) + string( "/kino/dvdauthor" );
+	loadTools( alternate_dir );
+
+	char *kinoHome=getenv("KINO_HOME");
+
+	if(kinoHome)
+	{
+		string alternate_dir = string( kinoHome ) + string( "/dvdauthor" );
+		loadTools( alternate_dir );
+	}
+
+	activateTool( prefs.exportMjpegDvdTool );
+
 	/* Get pointers to own controls */
 	fileEntry
 	= GTK_ENTRY( lookup_widget( common->getWidget(), "entry_export_mjpeg_file" ) );
@@ -86,12 +172,7 @@ ExportMJPEG::ExportMJPEG( PageExport *_exportPage, KinoCommon *_common ) :
 	mjpegAspect
 	= GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
 	                                      "optionmenu_export_aspect_ratio" ) ) ) );
-	mjpegXMLScript
-	= GTK_ENTRY( lookup_widget( exportPage->getWidget(),
-	                            "entry_export_mjpeg_script" ) );
-
 	cerr << "> Initializing MJPEG Export Page settings from Preferences" << endl;
-	gtk_entry_set_text( mjpegXMLScript, prefs.exportMjpegXMLScript  );
 	gtk_option_menu_set_history( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
 								"optionmenu_export_mjpeg_format" ) ), prefs.exportMjpegFormat );
 	gtk_option_menu_set_history( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
@@ -142,13 +223,15 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	cerr << ">>> ExportMJPEG::startExport" << endl;
 
 	// Set defaults on own controls
-	strcpy( prefs.exportMjpegXMLScript , gtk_entry_get_text( mjpegXMLScript ) );
 	prefs.exportMjpegFormat = g_list_index ( GTK_MENU_SHELL ( mjpegFormat ) ->children, gtk_menu_get_active( mjpegFormat ) );
 	prefs.exportMjpegDeinterlace = g_list_index ( GTK_MENU_SHELL ( deinterlaceMenu ) ->children, gtk_menu_get_active( deinterlaceMenu ) );
 	prefs.exportMjpegAspect = g_list_index ( GTK_MENU_SHELL ( mjpegAspect ) ->children, gtk_menu_get_active( mjpegAspect ) );
 	prefs.exportMjpegSceneSplit = gtk_toggle_button_get_active( splitCheck );
 	prefs.exportMjpegCleanup = gtk_toggle_button_get_active( cleanupCheck );
 	prefs.exportMjpegOutputDVDAuthorXML = gtk_toggle_button_get_active( authorCheck );
+	GtkWidget* widget = lookup_widget( common->getWidget(), "optionmenu_dvd_tools" );
+	widget = gtk_option_menu_get_menu( GTK_OPTION_MENU( widget ) );
+	prefs.exportMjpegDvdTool = g_list_index ( GTK_MENU_SHELL( widget )->children, gtk_menu_get_active( GTK_MENU( widget ) ) );
 
 	// TODO: Fix this buffer stuff.
 	gchar *filename;
@@ -180,10 +263,6 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	short channels = info.channels;
 	int frequency = info.frequency;
 
-	// Force audio resampling to that of the first frame
-	AudioResample<int16_ne_t,int16_le_t> *resampler = AudioResampleFactory<int16_ne_t,int16_le_t>::createAudioResample(
-	                               AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, frequency );
-
 	// vars for collecting audio and video pipe commands
 	bool split = false;
 	bool author = false;
@@ -193,8 +272,6 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	gchar *videopipe = NULL;
 	gchar *multiplex = NULL;
 	gchar *cleanup = NULL;
-	gchar *authorScript = NULL;
-	gchar *authorCommand = NULL;
 
 	// list to collect the names of the generated mpeg2 files
 	// this collection only proceeds if we are authoring an
@@ -226,7 +303,6 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	if( videoformat == 6 )
 	{
 		author = gtk_toggle_button_get_active( authorCheck );
-		authorScript = g_strdup( gtk_entry_get_text( mjpegXMLScript ));
 	}
 	active_item = gtk_menu_get_active( mjpegAspect );
 	aspectRatio = g_list_index ( GTK_MENU_SHELL ( mjpegAspect ) ->children, active_item );
@@ -286,8 +362,7 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 		case 6:
 			videoformat = 8;
 			strcpy( scale, "" );
-			if ( !strstr( audioencode, " -r " ) )
-				sprintf( audioOptions, "%s -r 48000", audioOptions );
+			frequency = 48000;
 			if ( !strstr( audioencode, " -b " ) )
 				sprintf( audioOptions, "%s -b 192", audioOptions );
 			break;
@@ -383,17 +458,37 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 
 		int imagesize = width * height * 3;
 
-		/* Iterate over frames in scene */
-		for ( int i = sceneBegin; i <= sceneEnd && exportPage->isExporting;
-		        i += every )
-		{
-			/* Call innerLoopUpdate */
-			innerLoopUpdate( i, begin, end, every );
+		// first call to innerLoopUpdate initializes progress tracker, which
+		// we want to do because calculateAdjustedRate generates paused time.
+		innerLoopUpdate( sceneBegin, begin, end, every );
 
+		// Determine correct amount of audio for duration
+		double adjustedRate = calculateAdjustedRate( playlist, frequency, sceneBegin, sceneEnd, every );
+		if ( !adjustedRate )
+			status = EXPORT_RESULT_ABORT;
+
+		// Setup a resampler
+		AsyncAudioResample<int16_ne_t,int16_le_t>* resampler = new AsyncAudioResample<int16_ne_t,int16_le_t>(
+			AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, playlist, frequency, sceneBegin, sceneEnd, every );
+		if ( resampler->IsError() )
+		{
+			std::cerr << ">>> Resampler error: " << resampler->GetError() << std::endl;
+			exportPage->isExporting = false;
+			status = EXPORT_RESULT_FAILURE;
+			break;
+		}
+
+		/* Iterate over frames in scene */
+		for ( int i = sceneBegin, j = 0; i <= sceneEnd && exportPage->isExporting;
+			i += every, j++ )
+		{
+			innerLoopUpdate( i, begin, end, every );
 			playlist->GetFrame( i, *frame );
 
-			resampler->Resample( *frame );
-			if ( !mpeg->OutputAudioFrame( reinterpret_cast<int16_t *>(resampler->output), resampler->size ) )
+			int requestedSamples = frame->CalculateNumberSamples( frequency, j );
+			int nsamples = resampler->Process( adjustedRate, requestedSamples );
+			if ( nsamples > 0 && !mpeg->OutputAudioFrame( reinterpret_cast<int16_t *>(resampler->GetOutput()), 
+				nsamples * channels * sizeof(int16_t) ) )
 			{
 				modal_message( _( "Error writing to KINO/MJPEG audio filter - aborting." ) );
 				status = EXPORT_RESULT_FAILURE;
@@ -413,10 +508,6 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 					break;
 				}
 			}
-
-			/* Update progressbar ... */
-			exportPage->updateProgress( ( gfloat ) ( i + every - begin )
-			                            / ( gfloat ) ( end + 1 - begin ) );
 		}
 
 		if ( status == EXPORT_RESULT_SUCCESS && !exportPage->isExporting )
@@ -428,8 +519,10 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 		// Try to finish the stream to create a valid mpeg if possible
 		mpeg->CloseAudio();
 		mpeg->CloseVideo();
+		delete resampler;
+		resampler = 0;
 
-		if ( multiplex != NULL && strcmp( multiplex, "" ) )
+		if ( status == EXPORT_RESULT_SUCCESS && multiplex != NULL && strcmp( multiplex, "" ) )
 		{
 			// expand the template to accomodate the scene splits
 			full = g_strdup_printf( multiplex, sceneString );
@@ -443,20 +536,23 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 		}
 
 		if ( Usercleanup )
-				mpeg->ExecuteCommand( cleanup );
+			mpeg->ExecuteCommand( cleanup );
 		
 		// Move to start of next scene
 		sceneBegin = sceneEnd + 1;
 	}
 
-	if ( author )
+	if ( status == EXPORT_RESULT_SUCCESS && author )
 	{
      	createAuthorXml( filename, nameList, split, isPAL );
-		if(strlen(authorScript))
+
+		// Run the selected tool
+		DVDTool *tool = m_dvdTools[ prefs.exportMjpegDvdTool ];
+		if ( tool )
 		{
-			cerr << ">>> Executing script: '" << authorScript << "' on file '" << filename << "'" << endl;
-			authorCommand = g_strdup_printf( "%s %s-dvdauthor.xml", authorScript, filename );
-			mpeg->ExecuteCommand( authorCommand );
+			string dvdauthorFile( string( filename ) + "-dvdauthor.xml" );
+			tool->execute( dvdauthorFile, string( filename ) );
+			tool->close();
 		}
 	}
 
@@ -465,12 +561,9 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	g_free( videopipe );
 	g_free( multiplex );
 	g_free( cleanup );
-	g_free( authorCommand );
-	g_free( authorScript );
 	delete mpeg;
 	delete video;
 	delete wav;
-	delete resampler;
 	delete[] pixels;
 
 	GetFramePool()->DoneWithFrame( frame );
@@ -570,6 +663,62 @@ ExportMJPEG::createAuthorXml( const char *filename,
 	out.close(); 
 }
 
+void ExportMJPEG::loadTools( string directory )
+{
+	char * filename;
+	struct dirent *entry;
+
+	DIR *dir = opendir( directory.c_str( ) );
+
+	if ( dir )
+	{
+		while ( ( entry = readdir( dir ) ) != NULL )
+		{
+			filename = g_strdup_printf( "%s/%s", directory.c_str( ), entry->d_name );
+			if ( entry->d_name[ 0 ] != '.' )
+			{
+				fprintf( stderr, "%s\n", filename );
+				DVDTool *tool = new DVDTool( filename );
+				if ( tool->m_active )
+					m_dvdTools.push_back( tool );
+				else
+					delete tool;
+			}
+			g_free( filename );
+		}
+		closedir( dir );
+	}
+	std::sort( m_dvdTools.begin(), m_dvdTools.end(), DVDTool::compare );
+}
+
+void ExportMJPEG::activateTool( int index )
+{
+	// Get the main widget
+	GtkWidget * window = common->getWidget();
+
+	// Start with the option menu
+	GtkOptionMenu *option_menu = GTK_OPTION_MENU( lookup_widget( window, "optionmenu_dvd_tools" ) );
+	GtkMenu *menu = GTK_MENU( gtk_option_menu_get_menu( option_menu ) );
+
+	// Create a menu if we don't have one
+	if ( menu )
+	{
+		menu = GTK_MENU( gtk_menu_new( ) );
+		for ( unsigned int i = 0; i < m_dvdTools.size( ); i ++ )
+		{
+			GtkWidget *item = gtk_menu_item_new_with_label( m_dvdTools[ i ] ->m_description.c_str( ) );
+			gtk_widget_show( item );
+			gtk_menu_append( menu, item );
+		}
+
+		gtk_option_menu_set_menu( option_menu, GTK_WIDGET( menu ) );
+	}
+
+	// Set the active item
+	gtk_option_menu_set_history( option_menu, index );
+}
+
+
 extern "C"
 {
 	void
