@@ -30,6 +30,9 @@
 #include "page_export.h"
 #include "kino_common.h"
 
+
+class DVDTool;
+
 /** This class represents the Export/Still Frames notebook page.
 */
 
@@ -52,6 +55,7 @@ private:
 	GtkEntry *mjpegXMLScript;
 
 	bool hasYuvDeinterlace;
+	vector < DVDTool * > m_dvdTools;
 
 	enum export_result
 	doExport( PlayList * playlist, int begin, int end, int every,
@@ -64,5 +68,9 @@ private:
 public:
 	ExportMJPEG( PageExport*, KinoCommon* );
 	virtual ~ExportMJPEG();
+
+	void loadTools( string directory );
+	void activateTool( int tool );
+	void selectTool( int tool );
 };
 #endif
