@@ -37,105 +37,12 @@ using std::endl;
 #include "preferences.h"
 #include "message.h"
 #include "frame.h"
+#include "rwpipe.h"
 
 #include <sys/types.h>
 #include <dirent.h>
 #include <sys/wait.h>
 
-class RWPipe
-{
-private:
-	int m_pid;
-
-protected:
-	int m_reader;
-	int m_writer;
-	GError *m_error;
-
-public:
-	RWPipe( ) : m_pid( -1 ), m_error( NULL )
-	{}
-
-	~RWPipe( )
-	{
-		stop( );
-	}
-
-	bool run( string command )
-	{
-		char * args[ 4 ];
-
-		args[ 0 ] = "/bin/sh";
-		args[ 1 ] = "-c";
-		args[ 2 ] = ( char * ) command.c_str( );
-		args[ 3 ] = NULL;
-
-		return g_spawn_async_with_pipes( ".", args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, &m_reader, NULL, &m_error );
-	}
-
-	bool isRunning( )
-	{
-		return m_pid != -1;
-	}
-
-	int readData( void *data, int size )
-	{
-		if ( m_pid != -1 )
-		{
-			int bytes = 0;
-			int len;
-			uint8_t *p = ( uint8_t * ) data;
-
-			while ( size > 0 )
-			{
-				len = read( m_reader, p, size );
-				if ( len <= 0 )
-					break;
-				p += len;
-				size -= len;
-				bytes += len;
-			}
-
-			return bytes;
-		}
-		else
-			return -1;
-	}
-
-	int readLine( char *text, int max )
-	{
-		int len = -1;
-		strcpy( text, "" );
-		if ( m_pid != -1 )
-		{
-			while ( len < max - 1 && readData( &text[ ++ len ], 1 ) )
-				if ( text[ len ] == '\n' )
-					break;
-			text[ len ] = '\0';
-		}
-		return len;
-	}
-
-	int writeData( void *data, int size )
-	{
-		if ( m_pid != -1 )
-			return write( m_writer, ( uint8_t * ) data, size );
-		else
-			return -1;
-	}
-
-	void stop( )
-	{
-		if ( m_pid != -1 )
-		{
-			close( m_reader );
-			close( m_writer );
-			waitpid( m_pid, NULL, 0 );
-			m_pid = -1;
-		}
-	}
-};
-
 class DVPipeTool : protected RWPipe
 {
 public:
@@ -234,11 +141,19 @@ ExportPipe::ExportPipe( PageExport *_exportPage, KinoCommon *_common ) :
 		Export( _exportPage, _common )
 {
 	cerr << "> Creating ExportPipe Page" << endl;
-	loadTools( DATADIR "/kino/scripts/exports" );
 
+	loadTools( DATADIR "/kino/scripts/exports" );
 	string alternate_dir = string( getenv( "HOME" ) ) + string( "/kino/exports" );
 	loadTools( alternate_dir );
 
+	char *kinoHome=getenv("KINO_HOME");
+
+	if(kinoHome)
+	{
+		string alternate_dir = string( kinoHome ) + string( "/exports" );
+		loadTools( alternate_dir );
+	}
+
 	GtkWidget *window = common->getWidget();
 	GtkOptionMenu *option_menu = GTK_OPTION_MENU( lookup_widget( window, "optionmenu_pipe_tools" ) );
 	g_signal_connect( G_OBJECT( option_menu ), "changed", G_CALLBACK( on_tool_change ), this );
@@ -382,8 +297,9 @@ void ExportPipe::clean()
 
 enum export_result ExportPipe::doExport( PlayList *playlist, int begin, int end, int every, bool preview )
 {
-	int i = -1;
+	enum export_result status = EXPORT_RESULT_SUCCESS;
 	Frame& frame = *GetFramePool()->GetFrame();
+	int i = -1, j;
 
 	// Get the main widget
 	GtkWidget *window = common->getWidget();
@@ -414,9 +330,6 @@ enum export_result ExportPipe::doExport( PlayList *playlist, int begin, int end,
 		if ( info.frequency == 0 )
 			info.frequency = 48000;
 
-		// Create the resampler
-		AudioResample<int16_ne_t,int16_le_t> *resampler = AudioResampleFactory<int16_ne_t,int16_le_t>::createAudioResample( AUDIO_RESAMPLE_SRC_SINC_MEDIUM_QUALITY, info.frequency );
-
 		// Evaluate argument for tool
 		string normalisation = frame.IsPAL( ) ? "pal" : "ntsc";
 		double length = ( double ) ( end - begin ) / ( frame.IsPAL( ) ? 25.0 : 29.97 );
@@ -428,7 +341,16 @@ enum export_result ExportPipe::doExport( PlayList *playlist, int begin, int end,
 		// Get the file name
 		string filename = ( char* ) gtk_entry_get_text( GTK_ENTRY( lookup_widget( window, "entry_export_pipe_file" ) ) );
 	
-		for (int pass = 1; pass <= passes; pass++ )
+		// first call to innerLoopUpdate initializes progress tracker, which
+		// we want to do because calculateAdjustedRate generates paused time.
+		innerLoopUpdate( begin, begin, end, every );
+
+		// Determine correct amount of audio for duration
+		double adjustedRate = calculateAdjustedRate( playlist, info.frequency, begin, end, every );
+		if ( !adjustedRate )
+			status = EXPORT_RESULT_ABORT;
+
+		for (int pass = 1; pass <= passes && exportPage->isExporting; pass++ )
 		{
 			// Execute the tool
 			bool success = false;
@@ -449,43 +371,54 @@ enum export_result ExportPipe::doExport( PlayList *playlist, int begin, int end,
 				success = tool->execute( normalisation, length, profile, filename, common->getPlayList()->GetDocName(), pass );
 			}
 	
+			// Create the resampler
+			AsyncAudioResample<int16_ne_t,int16_le_t>* resampler = new AsyncAudioResample<int16_ne_t,int16_le_t>(
+				AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, playlist, info.frequency, begin, end, every );
+			if ( resampler->IsError() )
+			{
+				std::cerr << ">>> Resampler error: " << resampler->GetError() << std::endl;
+				exportPage->isExporting = false;
+				status = EXPORT_RESULT_FAILURE;
+			}
+	
 			// Iterate over all frames in selection
-			for ( i = begin; success && i <= end && exportPage->isExporting; i += every )
+			for ( i = begin, j = 0; success && i <= end && exportPage->isExporting; i += every, j++ )
 			{
 				// Call innerLoopUpdate
-				innerLoopUpdate( i, begin, end, every );
-	
-				// Get the frame
-				playlist->GetFrame( i, frame );
+				if ( passes )
+					innerLoopUpdate( i, begin, end, every, pass == 1 ? _("Exporting (pass 1)") : _("Exporting (pass 2)") );
+				else
+					innerLoopUpdate( i, begin, end, every );
 	
 				// Resample
-				resampler->Resample( frame );
-				info.samples = resampler->size / info.channels / 2;
-				int16_t *p = reinterpret_cast<int16_t *>(resampler->output);
+				int requestedSamples = frame.CalculateNumberSamples( info.frequency, j );
+				info.samples = resampler->Process( adjustedRate, requestedSamples );
+				int16_t *p = reinterpret_cast<int16_t *>( resampler->GetOutput() );
 				for ( int s = 0; s < info.samples; s++ )
 					for ( int c = 0; c < info.channels; c++ )
 						audio_buffers[ c ][ s ] = *p++;
-				frame.EncodeAudio( info, ( int16_t ** ) audio_buffers );
+
+				// Get the frame
+				playlist->GetFrame( i, frame );
+	
+				if ( info.samples )
+					frame.EncodeAudio( info, audio_buffers );
 	
 				// Write the frame
 				success = tool->output( frame );
 			}
-			// Close the tool
 			tool->close( );
+			delete resampler;
 		}
-		// Delete the resampler
-		delete resampler;
 	}
 
 	GetFramePool()->DoneWithFrame( &frame );
 
 	// Return result
-	if ( !exportPage->isExporting )
+	if ( status == EXPORT_RESULT_SUCCESS && !exportPage->isExporting )
 		return EXPORT_RESULT_ABORT;
-	else if ( i > end )
-		return EXPORT_RESULT_SUCCESS;
-	else
-		return EXPORT_RESULT_FAILURE;
+
+	return status;
 }
 
 extern "C"
