@@ -23,6 +23,16 @@
 #include "kino_common.h"
 #include "page.h"
 
+#include <pthread.h>
+
+enum TimelineStatus {
+    TIMELINE_INIT,
+    TIMELINE_IDLE,
+    TIMELINE_STARTING,
+    TIMELINE_RUNNING,
+    TIMELINE_RESTART
+};
+
 /** This class defines the Page behaviour of the Timeline notebook page.
 */
 
@@ -32,11 +42,17 @@ private:
 	KinoCommon *common;
 	bool refresh_required;
 	int action;
-	vector< GdkPixbuf * > icons;
-	pthread_mutex_t key;
 	int last_begin;
 	int last_end;
 	int scene;
+	GtkIconView *view;
+	GtkListStore *model;
+
+	pthread_t thread;
+	pthread_mutex_t key;
+	pthread_mutex_t mutex;
+	enum TimelineStatus showIconsRunning;
+	static void* ThreadProxy( void *arg );
 
 public:
 	PageTimeline( KinoCommon *common );
@@ -50,6 +66,15 @@ public:
 	gboolean processKeyboard( GdkEventKey *event );
 	gboolean processCommand( char *cmd );
 	void timeFormatChanged();
+	GtkListStore *getModel() const
+	{
+		return model;
+	}
+	GtkIconView *getView() const
+	{
+		return view;
+	}
+	void* Thread();
 };
 
 
