@@ -79,7 +79,7 @@ extern "C"
 #define PLAYBACK_FRAMES 50
 
 	static Frame *frameContent[ PLAYBACK_FRAMES ];
-	extern dv1394Writer *dv1394;
+	extern IEEE1394Writer *writer1394;
 
 	/* signal callbacks */
 	gboolean
@@ -478,7 +478,7 @@ extern "C"
 */
 GArray *scenes = NULL;
 PageTrim::PageTrim( KinoCommon *common ) :
-		start_orig( 0 ), end_orig( 0 ), in_orig( 0 ), out_orig( 0 ), in( 0 ), out( 0 ),
+		start_orig( 0 ), update_start(0), end_orig( 0 ), in_orig( 0 ), out_orig( 0 ), in( 0 ), out( 0 ),
 		max( 0 ), pos( -1 ), currentScene( 0 ), lastPos( -1 ), mode( PAGE_TRIM_MODE_UNKNOWN ),
 		changed( false )
 {
@@ -494,7 +494,6 @@ PageTrim::PageTrim( KinoCommon *common ) :
 	loop_toggle = GTK_TOGGLE_BUTTON( lookup_widget( common->getWidget(), "togglebutton_trim_loop" ) );
 
 	this->g_copiedPlayList = new PlayList();
-	dv1394 = NULL;
 
 	// Attach the custom GTK+ Trim control
 	trim_adj[ TRIM_ADJ_POS ] = GTK_ADJUSTMENT(
@@ -544,22 +543,35 @@ void PageTrim::start()
 {
 	cerr << ">> Starting Trimmer" << endl;
 
-	start_orig = end_orig = in = in_orig = out = out_orig = max = 0;
+	start_orig = update_start = end_orig = in = in_orig = out = out_orig = max = 0;
 
 	for ( int i = 0; i < PLAYBACK_FRAMES; i ++ )
 		frameContent[ i ] = GetFramePool( ) ->GetFrame( );
 
 	this->displayer = new FrameDisplayer();
-	if ( Preferences::getInstance().dvExportDevice > 0
-	        && Preferences::getInstance().dv1394Preview )
-		dv1394 = new dv1394Writer(
-		             Preferences::getInstance().dvExportDevice,
-		             Preferences::getInstance().channel,
-		             Preferences::getInstance().dvExportBuffers,
-		             Preferences::getInstance().cip_n,
-		             Preferences::getInstance().cip_d,
-		             Preferences::getInstance().syt_offset
-		         );
+	if ( Preferences::getInstance().dv1394Preview )
+	{
+#ifdef HAVE_IEC61883
+		AVC avc;
+		Preferences::getInstance( ).phyID = avc.getNodeId( Preferences::getInstance( ).avcGUID );
+		Preferences::getInstance( ).phyID = avc.isPhyIDValid( Preferences::getInstance( ).phyID );
+		if ( Preferences::getInstance( ).phyID >= 0 && !writer1394 )
+			writer1394 = new iec61883Writer(
+			             avc.getPort(),
+			             Preferences::getInstance().channel,
+			             2 );
+#endif
+#ifdef HAVE_DV1394
+		if ( Preferences::getInstance().dvExportDevice > 0 && !writer1394 )
+			writer1394 = new dv1394Writer(
+			             Preferences::getInstance().dvExportDevice,
+			             Preferences::getInstance().channel,
+			             2,
+			             Preferences::getInstance().cip_n,
+			             Preferences::getInstance().cip_d,
+			             Preferences::getInstance().syt_offset );
+#endif
+	}
 
 	if ( ! Preferences::getInstance().trimModeInsert && common->g_currentFrame > -1 )
 		setMode( PAGE_TRIM_MODE_UPDATE );
@@ -621,8 +633,8 @@ void PageTrim::clean()
 	cerr << ">> Leaving Trimmer" << endl;
 	stopNavigator();
 	delete this->displayer;
-	if ( dv1394 != NULL )
-		delete dv1394;
+	delete writer1394;
+	writer1394 = NULL;
 	saveScene();
 	for ( int i = 0; i < PLAYBACK_FRAMES; i ++ )
 		GetFramePool( ) ->DoneWithFrame( frameContent[ i ] );
@@ -647,7 +659,7 @@ void PageTrim::loadScene( int newScene )
 
 	// first, save begin and end time markers to region of the
 	// the whole movie that this scene represents
-	start_orig = pl->FindStartOfScene( currentScene );
+	update_start = start_orig = pl->FindStartOfScene( currentScene );
 	end_orig = pl->FindEndOfScene( currentScene );
 
 	// get a local playlist representing just the scene
@@ -698,7 +710,6 @@ void PageTrim::loadScene( int newScene )
 	adjust->upper = max;
 	adjust->value = in;
 	g_signal_emit_by_name( adjust, "changed" );
-	skipPosUpdate = TRUE;
 	gtk_spin_button_set_value( spin_in, ( gfloat ) in );
 
 	adjust = gtk_spin_button_get_adjustment( spin_out );
@@ -706,7 +717,6 @@ void PageTrim::loadScene( int newScene )
 	adjust->upper = max;
 	adjust->value = out;
 	g_signal_emit_by_name( adjust, "changed" );
-	skipPosUpdate = TRUE;
 	gtk_spin_button_set_value( spin_out, ( gfloat ) out );
 
 	g_nav_ctl.escaped = TRUE;
@@ -790,7 +800,6 @@ bool PageTrim::loadFile( const string& filename )
 			adjust->upper = max;
 			adjust->value = in;
 			g_signal_emit_by_name( adjust, "changed" );
-			skipPosUpdate = TRUE;
 			gtk_spin_button_set_value( spin_in, ( gfloat ) in );
 
 			adjust = gtk_spin_button_get_adjustment( spin_out );
@@ -798,7 +807,6 @@ bool PageTrim::loadFile( const string& filename )
 			adjust->upper = max;
 			adjust->value = out;
 			g_signal_emit_by_name( adjust, "changed" );
-			skipPosUpdate = TRUE;
 			gtk_spin_button_set_value( spin_out, ( gfloat ) out );
 			
 			timeFormatChanged();
@@ -817,8 +825,6 @@ bool PageTrim::loadFile( const string& filename )
 		result = false;
 	}
 
-	currentScene = -1;
-
 	return result;
 }
 
@@ -884,7 +890,11 @@ void PageTrim::setMode( PageTrimMode newMode )
 			gtk_widget_hide( lookup_widget( common->getWidget(), "hbox_trim_insert" ) );
 
 			// Load the currently selected scene
-			loadScene( start_orig );
+			gfloat saved_in = in;
+			gfloat saved_out = out;
+			loadScene( update_start );
+			gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_IN ], saved_in );
+			gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_OUT ], saved_out );
 		}
 		else if ( newMode == PAGE_TRIM_MODE_INSERT )
 		{
@@ -1005,11 +1015,10 @@ void PageTrim::showFrame( int i, gboolean no_audio )
 			i = max;
 		playlist.GetFrame( i, frame );
 		common->showFrameInfo( i );
-		if ( dv1394 != NULL )
-			dv1394->SendFrame( frame );
+		if ( writer1394 != NULL )
+			writer1394->SendFrame( frame );
 		getFrameDisplayer() ->Put( frame, GTK_WIDGET( frameArea ), no_audio );
 		lastPos = i;
-		skipPosUpdate = TRUE;
 		gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_POS ], ( gfloat ) i );
 		GetFramePool( ) ->DoneWithFrame( &frame );
 		common->g_currentFrame = start_orig + (pos - in_orig);
@@ -1027,7 +1036,6 @@ void PageTrim::showFrame( int position, Frame& frame )
 	{
 		getFrameDisplayer() ->Put( frame, GTK_WIDGET( frameArea ), TRUE );
 		common->showFrameInfo( position );
-		skipPosUpdate = TRUE;
 		gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_POS ], ( gfloat ) position );
 	}
 }
@@ -1792,20 +1800,6 @@ gboolean PageTrim::processCommand( char *command )
 		cmd[ 0 ] = 0;
 	}
 
-	else if ( strcmp( real, "J" ) == 0 )
-	{
-		common->keyboardFeedback( cmd, _( "Join scenes" ) );
-		for ( ; count >= 1 && common->getPlayList() ->JoinScenesAt( common->g_currentFrame ); count -- )
-			common->hasListChanged = TRUE;
-		if ( common->hasListChanged == TRUE )
-		{
-			GetEditorBackup() ->Store( common->getPlayList() );
-			movedToFrame( pos );
-			common->getPageEditor()->DrawBar( common->g_currentFrame );
-			cmd[ 0 ] = 0;
-		}
-	}
-
 	// the following are specific to Trim
 	else if ( strcmp( cmd, "U" ) == 0 )
 	{
@@ -1864,19 +1858,34 @@ gboolean PageTrim::processCommand( char *command )
 		cmd[ 0 ] = 0;
 	}
 
+	else if ( strcmp( real, "J" ) == 0 )
+	{
+		common->keyboardFeedback( cmd, _( "Join scenes" ) );
+		for ( ; count >= 1 && common->getPlayList() ->JoinScenesAt( common->g_currentFrame ); count -- )
+			common->hasListChanged = TRUE;
+		if ( common->hasListChanged == TRUE )
+		{
+			GetEditorBackup() ->Store( common->getPlayList() );
+			movedToFrame( pos );
+			g_signal_emit_by_name( trim_adj[ TRIM_ADJ_POS ], "value-changed" );
+		}
+		cmd[ 0 ] = 0;
+	}
+
 	/* split scene */
 
 	else if ( strcmp( cmd, "Ctrl+J" ) == 0 && currentScene != -1 )
 	{
 		common->keyboardFeedback( cmd, _( "Split scene before frame" ) );
-		if ( common->getPlayList() ->SplitSceneBefore( common->g_currentFrame ) )
+		if ( pos >= in && pos <= out &&
+			common->getPlayList() ->SplitSceneBefore( common->g_currentFrame ) )
 		{
+			GetEditorBackup() ->Store( common->getPlayList() );
 			common->hasListChanged = TRUE;
 			movedToFrame( pos );
-			common->getPageEditor()->DrawBar( common->g_currentFrame );
-			GetEditorBackup() ->Store( common->getPlayList() );
-			cmd[ 0 ] = 0;
+			g_signal_emit_by_name( trim_adj[ TRIM_ADJ_POS ], "value-changed" );
 		}
+		cmd[ 0 ] = 0;
 	}
 
 	/* goto a frame */
@@ -1912,10 +1921,13 @@ gboolean PageTrim::processCommand( char *command )
 
 void PageTrim::windowMoved()
 {
-	if ( max == 0 )
-		common->clearPreview( frameArea );
-	else
-		showFrame( pos, TRUE );
+	if ( g_nav_ctl.active == FALSE )
+	{
+		if ( max == 0 )
+			common->clearPreview( frameArea );
+		else
+			showFrame( pos, TRUE );
+	}
 }
 
 
@@ -1940,7 +1952,7 @@ void PageTrim::showFrameInfo( int i )
 		string tc = "<span size=\"x-large\">" + common->getTime().parseFramesToString( i, common->getTimeFormat() ) + "</span>";
 		gtk_label_set_markup( positionLabelCurrent, tc.c_str() );
 		gtk_widget_set_redraw_on_allocate( GTK_WIDGET( positionLabelCurrent ), FALSE );
-		tc = _("Duration: ") + common->getTime().parseFramesToString( max + 1, common->getTimeFormat() );
+		tc = _("Duration: ") + common->getTime().parseFramesToString( out - in + 1, common->getTimeFormat() );
 		gtk_label_set_markup( positionLabelTotal, tc.c_str() );
 	}
 }
@@ -2085,8 +2097,8 @@ extern "C"
 				head = new_head;
 				pthread_mutex_unlock( &threadlock );
 
-				if ( dv1394 != NULL )
-					dv1394->SendFrame( *frameContent[ tail ] );
+				if ( writer1394 != NULL )
+					writer1394->SendFrame( *frameContent[ tail ] );
 
 				// Inform the audio thread that it's got something to work with
 				playing = true;
