@@ -871,8 +871,7 @@ bool PlayList::GetFrame( int frameNum, Frame &frame )
 		FileHandler *mediaFile = GetFileMap() ->GetMap() [ index ];
 		if ( data.clipFrame >= mediaFile->GetTotalFrames() )
 			data.clipFrame = mediaFile->GetTotalFrames() - 1;
-		mediaFile->GetFrame( frame, data.clipFrame );
-		return true;
+		return ( mediaFile->GetFrame( frame, data.clipFrame ) >= 0 );
 	}
 
 	return false;
