@@ -36,6 +36,23 @@ using std::make_pair;
 
 #define GKEYFILE_GROUP "general"
 
+static string kino_config_file()
+{
+	static string filename;
+
+	if(!filename.size())
+	{
+		char *kinoHome=getenv("KINO_HOME");
+
+		if(kinoHome)
+			filename=string(kinoHome)+string("/kinorc");
+		else
+			filename = string( getenv( "HOME" ) ) + string( "/.kinorc" );
+	}
+
+	return filename;
+}
+
 /** The global prefs object.
     
     It is accessible to other classes via the getInstance method.
@@ -111,7 +128,7 @@ Preferences::Preferences() : phyID( -1 )
 	string filename;
 	
 	// Give the new config file priority, but failover to the old gnome-config one
-	filename = string( getenv( "HOME" ) ) + string( "/.kinorc" );
+	filename = kino_config_file();
 	FILE *f = fopen( filename.c_str(), "r" );
 	if ( f )
 		fclose( f );
@@ -168,9 +185,9 @@ Preferences::Preferences() : phyID( -1 )
 	get( displayExtract, "displayExtract", "0" );
 	get( relativeSave, "relativeSave", "false" );
 	get( dvCaptureDriver, "dvCaptureDriver", "0" );
-	get( dvCaptureDevice, sizeof(dvCaptureDevice), "dvCaptureDevice", "/dev/ieee1394/dv/host0/PAL/in" );
-	get( dv1394Preview, "dv1394Preview", "false" );
-	get( dvExportDevice, sizeof(dvExportDevice), "dvExportDevice", "/dev/ieee1394/dv/host0/PAL/out" );
+	get( dvCaptureDevice, sizeof(dvCaptureDevice), "dvCaptureDevice", "/dev/dv1394/0" );
+	get( dv1394Preview, "dv1394Preview", "true" );
+	get( dvExportDevice, sizeof(dvExportDevice), "dvExportDevice", "/dev/dv1394/0" );
 	get( avcPollIntervalMs, "avcPollIntervalMs", "200" );
 	get( dvExportPrerollSec, "dvExportPrerollSec", "4" );
 	get( dvTwoPassEncoder, "dvTwoPassEncoder", "false" );
@@ -193,7 +210,7 @@ Preferences::Preferences() : phyID( -1 )
 	get( exportMjpegSceneSplit, "exportMjpegSceneSplit", "false" );
 	get( exportMjpegCleanup, "exportMjpegCleanup", "true" );
 	get( exportMjpegOutputDVDAuthorXML, "exportMjpegOutputDVDAuthorXML", "false" );
-	get( exportMjpegXMLScript, sizeof(exportMjpegXMLScript), "exportMjpegXMLScript", "" );
+	get( exportMjpegDvdTool, "exportMjpegDvdTool", "0" );
 
 	// build a metaValues key for each metaName
 	// metaValues is a range of values to pick from
@@ -716,10 +733,10 @@ Preferences::Save()
 	set( "exportMjpegSceneSplit", exportMjpegSceneSplit );
 	set( "exportMjpegCleanup", exportMjpegCleanup );
 	set( "exportMjpegOutputDVDAuthorXML", exportMjpegOutputDVDAuthorXML );
-	set( "exportMjpegXMLScript", exportMjpegXMLScript );
+	set( "exportMjpegDvdTool", exportMjpegDvdTool );
 
 	// Persist
-	string filename = string( getenv( "HOME" ) ) + string( "/.kinorc" );
+	string filename = kino_config_file();
 	FILE *f = fopen( filename.c_str(), "w" );
 	if ( f )
 	{
