@@ -20,6 +20,8 @@
 #include <config.h>
 #endif
 
+#define MAX_1394_PORTS 8
+
 #include "kino_common.h"
 #include "page.h"
 #include "preferences.h"
@@ -77,9 +79,9 @@ extern "C"
 		gPrefsDisplayMode = prefs.displayMode;
 
 		GtkWidget *dialog = lookup_widget( main_window, "preferences_dialog" );
+		gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(main_window));
 		widget = lookup_widget( dialog, "radiobutton_quicktime" );
 #ifdef HAVE_LIBQUICKTIME
-
 		gtk_widget_show( widget );
 		gtk_widget_set_sensitive( widget, 1 );
 #endif
@@ -159,18 +161,14 @@ extern "C"
 		widget = lookup_widget( dialog, "entry_size" );
 		sprintf( s, "%d", prefs.maxFileSize );
 		gtk_entry_set_text( GTK_ENTRY( widget ), s );
-		widget = lookup_widget( dialog, "entry_interface" );
-		sprintf( s, "%d", prefs.interface );
-		gtk_entry_set_text( GTK_ENTRY( widget ), s );
-		widget = lookup_widget( dialog, "entry_timing" );
-		sprintf( s, "%d", prefs.cip_n );
-		gtk_entry_set_text( GTK_ENTRY( widget ), s );
-		widget = lookup_widget( dialog, "entry_timing_threshold" );
-		sprintf( s, "%d", prefs.cip_d );
-		gtk_entry_set_text( GTK_ENTRY( widget ), s );
-		widget = lookup_widget( dialog, "entry_syt_offset" );
-		sprintf( s, "%d", prefs.syt_offset );
-		gtk_entry_set_text( GTK_ENTRY( widget ), s );
+
+#ifdef HAVE_IEC61883
+		gtk_widget_hide( lookup_widget( dialog, "label114" ) );
+		gtk_widget_hide( lookup_widget( dialog, "entry_dvcapture_device" ) );
+#else
+		widget = lookup_widget( dialog, "entry_dvcapture_device" );
+		gtk_entry_set_text( GTK_ENTRY( widget ), prefs.dvCaptureDevice );
+#endif
 		widget = lookup_widget( dialog, "checkbutton_enable_audio" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), prefs.enableAudio );
 		widget = lookup_widget( dialog, "entry_audio_device" );
@@ -294,12 +292,6 @@ extern "C"
 
 		widget = lookup_widget( dialog, "checkbutton_disable_key_repeat" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), prefs.disableKeyRepeat );
-		widget = lookup_widget( dialog, "entry_dvcapture_buffers" );
-		sprintf( s, "%d", prefs.dvCaptureBuffers );
-		gtk_entry_set_text( GTK_ENTRY( widget ), s );
-		widget = lookup_widget( dialog, "entry_dvexport_buffers" );
-		sprintf( s, "%d", prefs.dvExportBuffers );
-		gtk_entry_set_text( GTK_ENTRY( widget ), s );
 		widget = lookup_widget( dialog, "checkbutton_dv_clamp_luma" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), prefs.dvDecoderClampLuma );
 		widget = lookup_widget( dialog, "checkbutton_dv_clamp_chroma" );
@@ -314,73 +306,68 @@ extern "C"
 		gtk_option_menu_set_history( GTK_OPTION_MENU( widget ), prefs.displayExtract );
 		widget = lookup_widget( dialog, "checkbutton_relative" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), prefs.relativeSave );
-		widget = lookup_widget( dialog, "spinbutton_avc_interval" );
-		gtk_spin_button_set_value( GTK_SPIN_BUTTON( widget ), prefs.avcPollIntervalMs );
-		widget = lookup_widget( dialog, "spinbutton_dvexport_preroll" );
-		gtk_spin_button_set_value( GTK_SPIN_BUTTON( widget ), prefs.dvExportPrerollSec );
 
 		widget = lookup_widget( dialog, "optionmenu_avc_phyid" );
 
-#ifdef LIBRAW1394_OLD
-
-		if ( ( handle = raw1394_get_handle() ) )
-		{
-#else
 		if ( ( handle = raw1394_new_handle() ) )
 		{
-#endif
-			if ( raw1394_set_port( handle, prefs.interface ) >= 0 )
+			raw1394_portinfo ports[MAX_1394_PORTS];
+			int totalPorts = raw1394_get_port_info( handle, ports, MAX_1394_PORTS );
+			
+			gtk_widget_hide_all( lookup_widget( dialog, "hbox_ieee1394_warning" ) );
+			raw1394_destroy_handle( handle );
+			handle = NULL;
+
+			for (int port = 0; port < totalPorts; port++ )
 			{
-				gtk_widget_hide_all( lookup_widget( dialog, "hbox_ieee1394_warning" ) );
-
-				for ( currentNode = 0; currentNode < raw1394_get_nodecount( handle ); currentNode++ )
+				if ( ( handle = raw1394_new_handle_on_port( port ) ) )
 				{
-					if ( rom1394_get_directory( handle, currentNode, &rom1394_dir ) >= 0 )
+					for ( currentNode = 0; currentNode < raw1394_get_nodecount( handle ); currentNode++ )
 					{
-						if ( ( rom1394_get_node_type( &rom1394_dir ) == ROM1394_NODE_TYPE_AVC ) &&
-							avc1394_check_subunit_type( handle, currentNode, AVC1394_SUBUNIT_TYPE_VCR ) )
+						if ( rom1394_get_directory( handle, currentNode, &rom1394_dir ) >= 0 )
 						{
-							if ( rom1394_dir.label == NULL )
+							if ( ( rom1394_get_node_type( &rom1394_dir ) == ROM1394_NODE_TYPE_AVC ) &&
+								avc1394_check_subunit_type( handle, currentNode, AVC1394_SUBUNIT_TYPE_VCR ) )
 							{
-								sprintf( s, "Node %d", currentNode );
-								glade_menuitem = gtk_menu_item_new_with_label ( s );
-							}
-							else
-							{
-								glade_menuitem = gtk_menu_item_new_with_label ( rom1394_dir.label );
-							}
+								octlet_t currentGUID = rom1394_get_guid( handle, currentNode );
+								char currentGUIDStr[ 65 ];
 
-							octlet_t currentGUID = rom1394_get_guid( handle, currentNode );
-							char currentGUIDStr[ 65 ];
-							snprintf( currentGUIDStr, 64, "%08x%08x", ( quadlet_t ) ( currentGUID >> 32 ),
-						          ( quadlet_t ) ( currentGUID & 0xffffffff ) );
-
-							gtk_widget_show ( glade_menuitem );
-							gtk_menu_append ( GTK_MENU ( glade_menu ), glade_menuitem );
-							g_signal_connect ( G_OBJECT ( glade_menuitem ), "activate",
-						                     G_CALLBACK ( on_avc_phyid_activate ),
-						                     ( gpointer ) currentNode );
-
-
-							// default the phyID to the first AVC node found
-							if ( strcmp( prefs.avcGUID, "" ) == 0 )
-							{
-								prefs.phyID = currentNode;
-								strncpy( prefs.avcGUID, currentGUIDStr, 64 );
-								currentItem = itemCount;
+								snprintf( currentGUIDStr, 64, "%08x%08x", ( quadlet_t ) ( currentGUID >> 32 ),
+									( quadlet_t ) ( currentGUID & 0xffffffff ) );
+								currentGUIDStr[64] = 0;
+	
+								if ( rom1394_dir.label == NULL )
+									glade_menuitem = gtk_menu_item_new_with_label ( currentGUIDStr );
+								else
+									glade_menuitem = gtk_menu_item_new_with_label ( rom1394_dir.label );
+	
+								gtk_widget_show ( glade_menuitem );
+								gtk_menu_append ( GTK_MENU ( glade_menu ), glade_menuitem );
+								g_signal_connect ( G_OBJECT ( glade_menuitem ), "activate",
+												G_CALLBACK ( on_avc_phyid_activate ),
+												( gpointer ) strdup(currentGUIDStr) );
+								// TODO: free the duplicated string on re-entrance and destruction
+	
+								// default the phyID to the first AVC node found
+								if ( strcmp( prefs.avcGUID, "" ) == 0 )
+								{
+									prefs.phyID = currentNode;
+									strncpy( prefs.avcGUID, currentGUIDStr, 64 );
+									currentItem = itemCount;
+								}
+								else if ( strncmp( currentGUIDStr, prefs.avcGUID, 64 ) == 0 )
+								{
+									prefs.phyID = currentNode;
+									currentItem = itemCount;
+								}
+								itemCount++;
 							}
-							else if ( strncmp( currentGUIDStr, prefs.avcGUID, 64 ) == 0 )
-							{
-								prefs.phyID = currentNode;
-								currentItem = itemCount;
-							}
-							itemCount++;
+							rom1394_free_directory( &rom1394_dir );
 						}
-						rom1394_free_directory( &rom1394_dir );
 					}
+					raw1394_destroy_handle( handle );
+					handle = NULL;
 				}
-				raw1394_destroy_handle( handle );
-				handle = NULL;
 			}
 		}
 
@@ -397,15 +384,6 @@ extern "C"
 		widget = lookup_widget( dialog, "combo_entry_sample_rate" );
 		gtk_entry_set_text( GTK_ENTRY( widget ), prefs.v4lAudio );
 
-		widget = lookup_widget( dialog, "optionmenu_dvcapture_driver" );
-		gtk_option_menu_set_history( GTK_OPTION_MENU( widget ), prefs.dvCaptureDriver );
-		menuitem = GTK_MENU_ITEM ( gtk_menu_get_active ( GTK_MENU ( gtk_option_menu_get_menu( GTK_OPTION_MENU ( widget ) ) ) ) );
-		on_optionmenu_dvcapture_driver_activate( menuitem, NULL );
-		widget = lookup_widget( dialog, "entry_dvcapture_device" );
-		gtk_entry_set_text( GTK_ENTRY( widget ), prefs.dvCaptureDevice );
-		widget = lookup_widget( dialog, "entry_dvexport_device" );
-		gtk_entry_set_text( GTK_ENTRY( widget ), prefs.dvExportDevice );
-
 		widget = lookup_widget( dialog, "checkbutton_dv1394_preview" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), prefs.dv1394Preview );
 		widget = lookup_widget( dialog, "checkbutton_two_pass" );
@@ -420,6 +398,7 @@ extern "C"
 		gtk_widget_hide( lookup_widget( dialog, "checkbutton_two_pass" ) );
 #endif
 
+		gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_HELP, GTK_RESPONSE_OK, GTK_RESPONSE_CANCEL, -1);
 		gtk_widget_show( dialog );
 	}
 
@@ -427,25 +406,7 @@ extern "C"
 	void
 	on_avc_phyid_activate( GtkMenuItem * menu_item, gpointer user_data )
 	{
-		nodeid_t currentNode = ( nodeid_t )( long )user_data;
-		static raw1394handle_t handle;
-#ifdef LIBRAW1394_OLD
-
-		if ( ( handle = raw1394_get_handle() ) )
-		{
-#else
-		if ( ( handle = raw1394_new_handle() ) )
-		{
-#endif
-			if ( raw1394_set_port( handle, Preferences::getInstance().interface ) >= 0 )
-			{
-				octlet_t currentGUID = rom1394_get_guid( handle, currentNode );
-				snprintf( Preferences::getInstance().avcGUID, 64, "%08x%08x", ( quadlet_t ) ( currentGUID >> 32 ),
-				          ( quadlet_t ) ( currentGUID & 0xffffffff ) );
-			}
-		}
-		raw1394_destroy_handle( handle );
-		handle = NULL;
+		strncpy( Preferences::getInstance().avcGUID, (char *) user_data, 64 );
 	}
 
 	void
@@ -500,7 +461,7 @@ extern "C"
 		
 		gtk_widget_set_sensitive( widget, gtk_toggle_button_get_active( togglebutton ) );
 		if ( warn )
-			modal_message( _("You must restart Kino for V4L option to take effect.") );
+			modal_message_with_parent( _("You must restart Kino for V4L option to take effect."), widget );
 		warn = true;
 	}
 
@@ -519,22 +480,6 @@ extern "C"
 			JogShuttle::getInstance().stop();
 	}
 
-	/* **********************************************************************
-	 * Handling the mappings of the jogshuttle buttons
-	 * *********************************************************************/
-	/* This may be declared already in gtk, but may not */
-#if 0
-	gint gtk_option_menu_get_history( GtkWidget * option_menu )
-	{
-		GtkMenu * menu
-		= GTK_MENU ( GTK_OPTION_MENU ( option_menu ) ->menu );
-		GtkWidget * active_item
-		= gtk_menu_get_active ( menu );
-		return g_list_index ( GTK_MENU_SHELL ( menu ) ->children,
-		                      active_item );
-	}
-#endif
-
 	/* In this method, change the action as reflected in the option menu */
 	static void
 	on_optionmenu_jogshuttle_firstsecond_button_clicked( GtkButton * button,
@@ -727,7 +672,6 @@ extern "C"
 		prefs.autoSplit = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
 
 #ifdef HAVE_LIBQUICKTIME
-
 		widget = lookup_widget( dialog, "radiobutton_quicktime" );
 		if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) )
 			gPrefsSelectedFileFormat = QT_FORMAT;
@@ -761,8 +705,7 @@ extern "C"
 		prefs.maxFileSize = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
 		if ( prefs.maxFileSize < 0 )
 			prefs.maxFileSize = 2000;
-		widget = lookup_widget( dialog, "entry_interface" );
-		prefs.interface = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+#ifndef HAVE_LIBAVCODEC
 		GtkAdjustment *displayQualityValue;
 		widget = lookup_widget( dialog, "hscale_display_quality" );
 		displayQualityValue = gtk_range_get_adjustment( GTK_RANGE( widget ) );
@@ -771,12 +714,12 @@ extern "C"
 			prefs.displayQuality = int( gtk_adjustment_get_value( displayQualityValue ) );
 			common->moveToFrame();
 		}
-		widget = lookup_widget( dialog, "entry_timing" );
-		prefs.cip_n = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
-		widget = lookup_widget( dialog, "entry_timing_threshold" );
-		prefs.cip_d = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
-		widget = lookup_widget( dialog, "entry_syt_offset" );
-		prefs.syt_offset = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+#endif
+#ifdef HAVE_DV1394
+		widget = lookup_widget( dialog, "entry_dvcapture_device" );
+		strcpy( prefs.dvCaptureDevice, gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+		strcpy( prefs.dvExportDevice, gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+#endif
 		widget = lookup_widget( dialog, "checkbutton_enable_audio" );
 		prefs.enableAudio = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
 		widget = lookup_widget( dialog, "entry_audio_device" );
@@ -793,10 +736,6 @@ extern "C"
 
 		widget = lookup_widget( dialog, "checkbutton_disable_key_repeat" );
 		prefs.disableKeyRepeat = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
-		widget = lookup_widget( dialog, "entry_dvcapture_buffers" );
-		prefs.dvCaptureBuffers = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
-		widget = lookup_widget( dialog, "entry_dvexport_buffers" );
-		prefs.dvExportBuffers = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
 		widget = lookup_widget( dialog, "checkbutton_dv_clamp_luma" );
 		prefs.dvDecoderClampLuma = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
 		widget = lookup_widget( dialog, "checkbutton_dv_clamp_chroma" );
@@ -826,21 +765,12 @@ extern "C"
 		if ( prefs.defaultDirectory[ strlen( prefs.defaultDirectory ) - 1 ] != '/' )
 			strcat( prefs.defaultDirectory, "/" );
 
-		widget = gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( dialog, "optionmenu_dvcapture_driver" ) ) );
-		active_item = gtk_menu_get_active( GTK_MENU( widget ) );
-		prefs.dvCaptureDriver = g_list_index ( GTK_MENU_SHELL ( widget ) ->children, active_item );
-		widget = lookup_widget( dialog, "entry_dvcapture_device" );
-		strcpy( prefs.dvCaptureDevice, gtk_entry_get_text( GTK_ENTRY( widget ) ) );
-		widget = lookup_widget( dialog, "entry_dvexport_device" );
-		strcpy( prefs.dvExportDevice, gtk_entry_get_text( GTK_ENTRY( widget ) ) );
 		widget = lookup_widget( dialog, "checkbutton_dv1394_preview" );
 		prefs.dv1394Preview = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
-		widget = lookup_widget( dialog, "spinbutton_avc_interval" );
-		prefs.avcPollIntervalMs = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
-		widget = lookup_widget( dialog, "spinbutton_dvexport_preroll" );
-		prefs.dvExportPrerollSec = atoi( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+#ifndef HAVE_LIBAVCODEC
 		widget = lookup_widget( dialog, "checkbutton_two_pass" );
 		prefs.dvTwoPassEncoder = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+#endif
 		widget = lookup_widget( dialog, "checkbutton_expand_storyboard" );
 		prefs.expandStoryboard = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
 		widget = lookup_widget( dialog, "checkbutton_trim_mode_insert" );
@@ -868,28 +798,11 @@ extern "C"
 	}
 
 
-	void
-	on_optionmenu_dvcapture_driver_activate ( GtkMenuItem *menuitem, gpointer user_data )
-	{
-		GtkEntry *device;
-		gboolean activate = 1;
-
-		if ( GTK_WIDGET ( menuitem ) ==
-		     lookup_widget( GTK_WIDGET ( menuitem ),
-		                    "optionmenu_dvcapture_driver_raw1394") )
-			activate = 0;
-		
-		device = GTK_ENTRY ( lookup_widget( GTK_WIDGET ( menuitem ),
-		                     "entry_dvcapture_device") );
-		gtk_entry_set_editable( device, activate );
-		gtk_widget_set_child_visible( GTK_WIDGET ( device ), activate );
-	}
-
 	void
 	on_button_capture_open_clicked( GtkButton       *button,
 	                                gpointer         user_data )
 	{
-		char *filename = common->getFileToOpen( _("Choose a DV file") );
+		char *filename = common->getFileToOpen( _("Choose a DV file"), true, GTK_WIDGET(button) );
 		if ( filename && strcmp( filename, "" ) )
 		{
 			gtk_entry_set_text( GTK_ENTRY( lookup_widget( GTK_WIDGET( button ), "entry_file" ) ),
