@@ -49,6 +49,7 @@ using std::deque;
 class V4LStruct
 {
 public:
+	virtual ~V4LStruct() {}
 	virtual void *getStruct() = 0;
 };
 
@@ -58,6 +59,7 @@ public:
 class V4LDevice
 {
 public:
+	virtual ~V4LDevice() {}
 	virtual int getHandle() = 0;
 	bool request( int req, V4LStruct *v4l );
 	bool request( int req, void *addr );
@@ -135,8 +137,7 @@ public:
 };
 
 class V4LFrame : public V4LStruct
-	{}
-;
+{};
 
 class V4L : public V4LDevice
 {
